function [out, report] = FitLee_GISAXSvcutSBA_spherefit(varargin)
    % schultz polydisperse sphere with structure factors.
    % This function is only for fitting a set of data at a time.
    % if numel(varargin) == 1 and p is not a struct but a number, then
    % it generate set of default parameters for using this function.
FitLee_helpstr = {'GISAXS vertical linecut fit with SBA. ' ,...
'I(q) = Iq + background',...
'Parameters',...
'',...
'Note that',...
' ',...
'Byeongdu Lee (blee@anl.gov)',...
'Ref: ',...
'    Lee, B., et al. (2008). J. Appl. Cryst. 41(1): 134-142 '};

if numel(varargin) > 1
    p = varargin{1};
    q = varargin{2};
    isini = 0;
elseif numel(varargin) == 1
    p = varargin{1};
    isini = 1;
    if ishandle(p)
        guiadd(p);
        return
    end    
    if ischar(p)
        out = FitLee_helpstr;
        return
    end
elseif numel(varargin) == 0
    p = 1;
    isini = 1;
end

%% initialize fit parameter bestP
if isini
    Nf = p;
    bestP = [];
    bestP.I0 = 1;
    bestP.R = 100;
    bestP.sig = 10;
    bestP.B = 1;          % from 0 to 1
    bestP.cos_phaseB = 1; % from -1 to 1
    bestP.ai = 0.1;
    bestP.tthf = 0.05;
    bestP.Xeng = 14;
    bestP.filmeden = 3.66;
    bestP.filmbeta = 1E-7;
    bestP.filmthick = 100;
    bestP.Bck_I0 = 0;
    bestP.Bck_power = -2;
    bestP.Bck_const = 0;
    bestP.string = 'FitLee_GISAXSvcutSBA_spherefit(figH)';
    assignin('base', 'bestP', bestP);
    out = bestP;
    return
end

%% fitting codes .......................
if iscell(q);
    if numel(q) > 1
        error('FitLee_schultzsphere.m is for fitting a set of data, for now')
    end
    q = q{1};
end

% vertical cut.
if isfield(p, 'tthf')
    af = q;
    [Ti, Tf, Ri, Rf, kiz, kfz, ai, af, tthi, tthf, qx, qy, qp, qz] = ...
        GISAXS_wave_Amp_Vector(p.ai, af, 0, p.tthf, eng2wl(p.Xeng), ...
        [0, p.filmeden, 0.7], [0, p.filmbeta, 1E-7], [0, -1*p.filmthick], [0,0], 0, 2);
    qtz = real(qz{2, 1});
    qrz = real(qz{2, 2});

    [Iq, Iq2] = Iq_sphere_SBA(qp, qtz, qrz, p.R, p.sig, p.B, p.cos_phaseB);
end
% Horizontal cut
if isfield(p, 'af')
    tthf = q;
    [Ti, Tf, Ri, Rf, kiz, kfz, ai, af, tthi, tthf, qx, qy, qp, qz] = ...
        GISAXS_wave_Amp_Vector(p.ai, p.af, 0, tthf, eng2wl(p.Xeng), ...
        [0, p.filmeden, 0.7], [0, p.filmbeta, 1E-7], [0, -1*p.filmthick], [0,0], 0, 2);
    qtz = real(qz{2, 1});
    qrz = real(qz{2, 2});

    [Iq2, Iq] = Iq_sphere_SBA(qp, qtz, qrz, p.R, p.sig, p.B, p.cos_phaseB);
end
%Iq = Iq2;
back = p.Bck_I0./af.^p.Bck_power + p.Bck_const;
out = p.I0*Iq+back;
if isnan(out)
    out = ones(size(out));
end

if nargout == 2
    x = 0:1:(max(p.r0, p.r1)+max(p.sig0, p.sig1)*10);
    nr0 = schultz(p.r0, p.sig0, x);
    nr = nr0;
    if p.Nratio > 0
        nr1 = schultz(p.r1, p.sig1, x);
        nr = nr+p.Nratio*nr1;
    end
    figure;
    plot(x, nr);xlabel('Radius (A)');ylabel('n(r)')
    fprintf('Volume of particle0 : %0.3e A^3\n', V0);
    fprintf('Number fraction of particle0 : %0.3e particles/cm^3. \n', pnumberfraction)
    report = '';
end

function guiadd(figH)
    uph = findobj(figH, 'type', 'uipanel');
    pos = get(figH, 'position');
    hFigHeight = pos(end);
        uicontrol(...
          'style'                     , 'text', ...
          'backgroundcolor'           , [0,0.5,0.5], ...
          'foregroundcolor'           , [1,1,1], ...
          'horizontalalignment'       , 'Left', ...
          'parent'                    , uph, ...
          'string'                  ,   'Cut type',...
          'position'                  ,[1, hFigHeight-210,150,20], ...
          'tag'                       , 'text_Pshape');
    popup = uicontrol(...
          'parent', uph,...
          'horizontalalignment'       , 'Left', ...
          'Style', 'popup',...
          'String', {'Vertical','Horizontal'},...
          'Position', [100, hFigHeight-235, 100, 50],...
          'Callback', @setshape);
function setshape(varargin)
    obj = varargin{1};
    list = get(obj, 'string');
    switch list{get(obj, 'value')}
        case 'Vertical'
            o = findobj(gcbf, 'string', 'af');
            set(o, 'string', 'tthf');
        case 'Horizontal'
            o = findobj(gcbf, 'string', 'tthf');
            set(o, 'string', 'af');
    end
    