function ProjImage2Ewaldsphere(varargin)
% function ProjImage2Ewaldsphere(img, saxs)
% function ProjImage2Ewaldsphere(img, saxs, "property", value)
% function ProjImage2Ewaldsphere(img, center, psize, waveln, SDD, phi)
% function ProjImage2Ewaldsphere(img, center, psize, waveln, SDD, phi, "property", value)
% properties : "parent"
%
% Byeongdu Lee
% 8/7/2016

pProp = -1;
for i=1:numel(varargin)
    if ischar(varargin{i})
        pProp = i;
        break
    end
end
hadParentAsPVPair = false;
isROI = false;
if pProp > 0
    i_properties = varargin(pProp:end);
    varargin(pProp:end) = [];
    for i=1:numel(i_properties)
        if strncmpi(i_properties{i}, 'parent', length(i_properties{i}))
            cax = i_properties{i+1};
            hadParentAsPVPair = true;
        end
        if strncmpi(i_properties{i}, 'ROI', length(i_properties{i}))
            ROI = i_properties{i+1};% [Xmin, Xmax Ymin Ymax]
            ROIy = ROI(1):ROI(2);
            ROIz = ROI(3):ROI(4);
            isROI = true;
        end
    end
end

i_type = 0;
if numel(varargin) == 2
    img = double(varargin{1});
    saxs = varargin{2};
    center = saxs.center;
    i_type = 1;
elseif numel(varargin) == 6
    img = varargin{1};
    center = double(varargin{2});
    psize = varargin{3};
    waveln = varargin{4};
    SDD = varargin{5};
    phi = varargin{6};
    i_type = 2;
end

if ~isROI
    ROIz = 1:size(img, 1);
    ROIy = 1:size(img, 2);
else
    img = img(ROIz, ROIy);
end
[Y, Z] = meshgrid(ROIy, ROIz);

switch i_type
    case 1
        q = pixel2qv([Y(:), Z(:)], saxs);
        qx = q.qx;
        qy = q.qy;
        qz = q.qz;
    case 2
        Ym = Y - center(1);
        Zm = Z - center(2);
        [qx, qy, qz] = pixelNphi2qv(Zm(:), Ym(:), phi, psize, waveln, SDD);
    otherwise
        disp('Usage is not correct.')
end

qx = reshape(qx, size(img));
qy = reshape(qy, size(img));
qz = reshape(qz, size(img));

if hadParentAsPVPair
    surface(qx, qy, qz, img, 'parent', cax);
else
    figure;
    surface(qx, qy, qz, img, 'EdgeColor','none','FaceColor','texturemap');
end
xlabel('q_x');
ylabel('q_y');
zlabel('q_z');
