function fluxonsample = gisaxs_fluxonsample(varargin)
% Calculate the fractional fulx on sample
% see also gisaxs_rockinganalysis.m
% example
%   [p, beamprofile2] = gisaxs_rockinganalysis()
%   f = gisaxs_fluxonsample(p, beamprofile2, ai) or
%   f = gisaxs_fluxonsample(p, beamprofile2, Ibeam, [fullbeam, darkbeam])
if numel(varargin) == 3
    p = varargin{1};
    beamprofile2 = varargin{2};
    ai = varargin{3};
else
    p = varargin{1};
    beamprofile2 = varargin{2};
    Ibeam = varargin{3};
    beam = varargin{4};
    fullbeam = beam(1);
    darkbeam = beam(2);
    ai = NaN;
end


if isnan(ai)
    ang = linspace(p.a0, 1, 1000);
    sav_pos_ai = FitLee_rocking(p, ang);
    Ibeam = (Ibeam - darkbeam)/(fullbeam-darkbeam);
    z_up = interp1(beamprofile2(:,2), beamprofile2(:,1), Ibeam);
    ai = interp1(sav_pos_ai, ang, z_up)-p.a0;
    z_down = z_up - (p.Lu+p.Ld)*sin(ai*pi/180);
else
    z_up = p.Lu*sin(ai*pi/180)+p.zp;
    z_down = -p.Ld*sin(ai*pi/180)+p.zp;
end
if (z_down < beamprofile2(1,1))
    z_down = beamprofile2(1,1);
end
if (z_up > beamprofile2(end,1))
    z_up = beamprofile2(end,1);
end
%interp1(beamprofile2(:,1), beamprofile2(:,2), z_down)
%interp1(beamprofile2(:,1), beamprofile2(:,2), z_up)
fluxonsample = 0;
if ai>0
fluxonsample = interp1(beamprofile2(:,1), beamprofile2(:,2), z_down) - ...
    interp1(beamprofile2(:,1), beamprofile2(:,2), z_up);
end