function [T, error_file] = Cheader(filename, parameter_name)

fid = fopen(filename, 'r');
error_file = 0;

fseek(fid, 61, -1);
B = ferror(fid);
data_of_type = fread(fid, 1, 'int8');            % 61    : 1byte 

if data_of_type == 0
   da_type = 'int16';
elseif data_of_type == 1
   da_type =  'single';
else
   disp('data loading error')
   error_file = -1;
end


% header input

if nargin == 2
   
if parameter_name == 'yr'             % year
   parameter = 1;
elseif parameter_name == 'mm'    % month
   parameter = 2;
elseif parameter_name == 'dd'     % day 
   parameter = 3;
elseif parameter_name == 'hr'      % hour
   parameter = 4;
elseif parameter_name == 'mi'      % mi
   parameter = 5;
elseif parameter_name == 'se'      % se 
   parameter = 6;
elseif parameter_name == 'ex'       % exposure_time
   parameter = 7;
elseif parameter_name == 'i1'        % ic1
   parameter = 8;
elseif parameter_name == 'i2'        % ic2
   parameter = 9;
elseif parameter_name == 'sc'       % scintillation counter value
   parameter = 10;
elseif parameter_name == 'Tr'        % Transmittance
   parameter = 11;
elseif parameter_name == 'wl'        % wavelength
   parameter = 12;
elseif parameter_name == 'SD'       % SDD
   parameter = 13;
elseif parameter_name == 'bX'        % beamsize X
   parameter = 14;
elseif parameter_name == 'bY'        % beamsize Y
   parameter = 15;
elseif parameter_name == 'cX'        % center position X
   parameter = 16;
elseif parameter_name == 'cY'        % center position Y
   parameter = 17;
elseif parameter_name == 'qs'        % q scale factor
   parameter = 18;
elseif parameter_name == 'te'         % Temperature 
   parameter = 19;
elseif parameter_name == 'ti'          % thickness
   parameter = 20;
elseif parameter_name == 'ab'        % absolute intensity scaling factor
   parameter = 21;
elseif parameter_name == 'dt'         % datatype
   parameter = 22;
elseif parameter_name == 'cm'       % comment 
   parameter = 23;
elseif parameter_name == 'ps'       % pixel_size
   parameter = 24;
elseif parameter_name == 'vs'       % version
   parameter = 25;
elseif parameter_name == 'al'         % all parameter re input.
   parameter = 0;
else
   parameter = -1;
   disp('error occurred, input parameter_name is wrong')
end

else
   parameter = 0;
end            % if nargin == 2.........



if parameter == 0 | parameter == 1
   fseek(fid, 0, -1);
   B = ferror(fid);
   T.year = fread(fid, 1, 'int16');                 % 0-1 : 2byte
end


if parameter == 0 | parameter == 2
   fseek(fid, 2, -1);
   B = ferror(fid);
   T.month = fread(fid, 1, 'int8');            % 2    : 1byte
end


if parameter == 0 | parameter == 3
   fseek(fid, 3, -1);
   B = ferror(fid);
   T.day = fread(fid, 1, 'int8');                % 3    : 1byte
end


if parameter == 0 | parameter == 4
  fseek(fid, 4, -1);
  B = ferror(fid);
  T.hour = fread(fid, 1, 'int8');              % 4    : 1byte
end  


if parameter == 0 | parameter == 5
  fseek(fid, 5, -1);
  B = ferror(fid);
  T.minute = fread(fid, 1, 'int8');            % 5    : 1byte
end  


if parameter == 0 | parameter == 6
  fseek(fid, 6, -1);
  B = ferror(fid);
  T.second = fread(fid, 1, 'int8');            % 6    : 1byte
end  


if parameter == 0 | parameter == 7
  fseek(fid, 7, -1);
  B = ferror(fid);
  T.ex_time = fread(fid, 1, 'int16' );            % 7-8    : 2byte
end

  
if parameter == 0 | parameter == 8
     fseek(fid, 9, -1);
     B = ferror(fid);
     T.IC1 = fix(fread(fid, 1, 'single')*1000)/1000;     %Ҽ 3° ڸ        % 9-12    : 4byte
end  
  
  
if parameter == 0 | parameter == 9
   fseek(fid, 13, -1);
   B = ferror(fid);
   T.IC2 = fix(fread(fid, 1, 'single')*1000)/1000;       %Ҽ 3° ڸ      % 13-16    : 4byte
 end  
  
  
if parameter == 0 | parameter == 10
   fseek(fid, 17, -1);
   B = ferror(fid);
   T.Scin = fread(fid, 1, 'single');            % 17-20    : 4byte
end  
  
  
if parameter == 0 | parameter == 11
    fseek(fid, 21, -1);
    B = ferror(fid);    
    T.Tr = fix(fread(fid, 1, 'single')*1000)/1000;            % 21-24    : 4byte
end  


if parameter == 0 | parameter == 12
  fseek(fid, 25, -1);
  B = ferror(fid);
  T.wavelength = fix(fread(fid, 1, 'single')*1000)/1000;    %unit : Angstrom        % 25-28    : 4byte
end  
  
  
if parameter == 0 | parameter == 13
  fseek(fid, 29, -1);
  B = ferror(fid);
  T.SDD = fread(fid, 1, 'single');            % 29-32    : 4byte
end  


if parameter == 0 | parameter == 14
  fseek(fid, 33, -1);
  B = ferror(fid);  
  T.beamX = fread(fid, 1, 'int16');            % 33-34    : 2byte
end  


if parameter == 0 | parameter == 15
  fseek(fid, 35, -1);
  B = ferror(fid);
  T.beamY = fread(fid, 1, 'int16');            % 35-36    : 2byte
end  
  
  
if parameter == 0 | parameter == 16
     fseek(fid, 37, -1);
     B = ferror(fid);
     T.centerX = fread(fid, 1, 'single');            % 37-38    : 2byte
end  


if parameter == 0 | parameter == 17
  fseek(fid, 41, -1);
  B = ferror(fid);
  T.centerY = fread(fid, 1, 'single');            % 41-44    : 2byte
end  


if parameter == 0 | parameter == 18
  fseek(fid, 45, -1);
  B = ferror(fid);
  T.q_scale = fread(fid, 1, 'single');            % 45-48    : 4byte
end  


if parameter == 0 | parameter == 19
  fseek(fid, 49, -1);
  B = ferror(fid);
  T.temperature = fix(fread(fid, 1, 'single')*100)/100;            % 49-52    : 4byte
end  


if parameter == 0 | parameter == 20
  fseek(fid, 66, -1);
  B = ferror(fid);
  T.thick = fix(fread(fid, 1, 'single')*100)/100;    %unit : mm        % 66-69    : 4byte
end  




if parameter == 0 | parameter == 21
  fseek(fid, 70, -1);
  B = ferror(fid);
  T.Abs = fix(fread(fid, 1, 'single')*1000)/1000;            % 70-73    : 4byte
end  



if parameter == 0 | parameter == 22
   fseek(fid, 65, -1);
   B = ferror(fid);
   T.datatype_code = fread(fid, 1, 'int8');            % 65    : 1byte 
end  
  
  
if parameter == 0 | parameter == 23
   fseek(fid, 78, -1);
   B = ferror(fid);
   TTT = fread(fid, 80, 'char');       %76-157 : 80byte
   T.comment = setstr(TTT');
end

  
if parameter == 0 | parameter == 24
   fseek(fid, 74, -1);
   B = ferror(fid);
   T.pixel_size = fread(fid, 1, 'int16');      % 74-75 : 2byte   pixel size
end

  
if parameter == 0 | parameter == 25
   fseek(fid, 76, -1);
   B = ferror(fid);
   T.version = fread(fid, 1, 'int16');      % 76-77 : 2byte    version
end



fseek(fid, 53, -1);
T.sizeX = fread(fid, 1, 'int16');            % 43-54    : 2byte
fseek(fid, 55, -1);
T.sizeY = fread(fid, 1, 'int16');            % 55-56    : 2byte
fseek(fid, 76, -1);
T.version = fread(fid, 1, 'int16');            % 76-77    : 2byte
  
fclose(fid);
