function y = cal_dlul(q, coord, atm, Fq)

atmtype = unique(atm);
factor = 1;

if numel(atmtype) == 1
    ddl = distNzero(coord);
    [delta, w] = getdeltaw(max(q), factor);
    S = nofsample(ddl, delta, w);
    [ul, dl] = hist(ddl, S);
    y = rhs(q, 2*ul, dl, factor);
    y = numel(coord)/3+y(:);
    return;
else
    I0 = zeros(size(q));
    indx = cell(numel(atmtype),1);
    for i=1:numel(atmtype)
        indx{i} = find(atm == atmtype(i));
        I0 = I0 + abs(Fq(:, atmtype(i))).^2*numel(indx{i});
    end
    
    y = zeros(size(q));
    
    for m=1:numel(atmtype)
        for n=m:numel(atmtype)
            m_co = coord(indx{m}, :);
            n_co = coord(indx{n}, :);
            ddl = distNzero2(m_co, n_co);
            
            [delta, w] = getdeltaw(max(q), factor);
            S = nofsample(ddl, delta, w);
            [ul, dl] = hist(ddl, S);
            yrhs = rhs(q, ul, dl, factor);
            y = y + Fq(:,atmtype(m)).*Fq(:,atmtype(n)).*yrhs;
        end
    end
    y = I0 + y;
end
end

function y = rhs(q, ul, dl, factor)
    delta = dl(end) - dl(end-1);
    w = delta*(factor/2);
    Cuq = Cq(q, w);
    y = zeros(size(q));
    for i=1:numel(q)
        y(i) = Cuq(i).*sum(ul.*sinc(q(i)*dl));
    end
end
function [S,D] = nofsample(dl, delta, w)   % for a box function of U(x)
    D = max(dl);
    S = floor((D+w/2)/delta);
end

function [delta, w] = getdeltaw(qmax, factor)
    % factor should be bigger than 1;
    if nargin ==1
        factor = 1;
    end
    if factor < 1;
        disp('factor should be bigger than 1')
    end
    deltamax = 1/qmax/2;
    delta = deltamax/factor;
    w = delta*(factor/2); % w should be smaller than 2*deltamax and bigger than delta, for a box function of U(x).
end

function x = Cq(q, w)   % C fuction for a box function: Eq. 19
    x = (pi*q*w)./(sin(pi*q*w));
end