#include "mex.h"
#include <math.h>
#include <time.h>
#define MAXARRAY 100000

double sinc(double x)
{
if (x==0) return 1;
else return sin(x+0.000001)/(x+0.000001);
}

void mexFunction(int nlhs,       mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{
    /* Declare variables */
    FILE *pFile;
    long atm_len;
    mwSize q_pnt;
    double *pind;
    double *q, *fm;
    //double *q, *fm, *x, *y, *z;
    float *x, *y, *z, *atm;
    float xa, ya, za;
    int atmm;
    char *str[100];
//    double *atm;
    double fmj, fmk, r_jk;
/*    double p; */
/*    double r_jk=0; */
    long m, j, k, i;
    long kk;
    int eofcheck;
    double elapsed;
    clock_t start, end;
    
    /* Check for proper number of input and output arguments */    
    if (nrhs != 3) {
	mexErrMsgTxt("Five input arguments required. q(n), formfactor(n,an), x(an), y(an), z(an), atm(an)");
    } 
    if (nlhs > 1){
	mexErrMsgTxt("Too many output arguments.");
    }
    /* Check data type of input argument */
    if (!(mxIsDouble(prhs[0]))) {
      mexErrMsgTxt("Input array must be of type double.");
    }
    
    /* Get the number of elements in the input argument */

    /* Get the data */
    q = mxGetPr(prhs[0]);
    fm = mxGetPr(prhs[1]);
    atm_len = (long) (mxGetScalar(prhs[2]));
    printf("atm_len = %d", atm_len);
    q_pnt = mxGetNumberOfElements(prhs[0]);

    if (atm_len>MAXARRAY){
        printf("Increase MAXARRAY\n");
        return;
    }
    if (atm_len < MAXARRAY) {
        x = malloc(sizeof(float) * atm_len);
        y = malloc(sizeof(float) * atm_len);
        z = malloc(sizeof(float) * atm_len);
        atm = malloc(sizeof(float) * atm_len);
    } else {
        x = malloc(sizeof(float) * MAXARRAY);
        y = malloc(sizeof(float) * MAXARRAY);
        z = malloc(sizeof(float) * MAXARRAY);
        atm = malloc(sizeof(float) * MAXARRAY);
    }
    pFile = fopen("mydata.txt", "r");
    i = 0;
    eofcheck = 0;
    
    if (pFile != NULL) {
        while(eofcheck != EOF){
            eofcheck=fscanf(pFile, "%f %f %f %i", &xa, &ya, &za, &atmm);
            x[i] = xa;
            y[i] = ya;
            z[i] = za;
            atm[i] = atmm;
            i++;
        }
        fclose(pFile);
    }
    /* Get the number of dimensions in the input argument. Allocate the
       space for the return argument */
    plhs[0]=mxCreateDoubleMatrix(q_pnt,1,mxREAL);
    pind=mxGetPr(plhs[0]);
	
    /* Fill in the indices to return to MATLAB. This loops through the
     * elements and checks for non-zero values. If it finds a non-zero
     * value, it then calculates the corresponding MATLAB indice and
     * assigns them into the output array.  The 1 is added to the
     * calculated indice because MATLAB is 1 based and C is zero
     * based. */
    start = clock();
    for (m=0;m<q_pnt;m++) {
        pind[m] = 0;
        kk = (long) (m+q_pnt*(atm[atm_len-1]-1));
        fmj = fm[kk];
        pind[m] = pind[m] + fmj*fmj;
        for (j=0;j<atm_len-1;j++) {
            kk = (long) (m+q_pnt*(atm[j]-1));
            fmj = fm[kk];
            for (k=j+1;k<atm_len;k++) {
                kk = (long) (m+q_pnt*(atm[k]-1));
                fmk = fm[kk];
                r_jk = sqrt((x[j]-x[k])*(x[j]-x[k]) + (y[j]-y[k])*(y[j]-y[k]) + (z[j]-z[k])*(z[j]-z[k]));
                pind[m] += 2.0*fmj*fmk*sinc(q[m]*r_jk);
            }
            pind[m] = pind[m] + fmj*fmj;
        }
    }
    free(x);
    free(y);
    free(z);
    free(atm);
    end = clock();
    elapsed = ((double)(end-start))/ CLOCKS_PER_SEC;
    printf("time elapsed is %f\n", elapsed);
}