function y = debyem_fast(q, x, y, z, atm, Fq)
% debye fast calculator using histogram
% ref. A. Cervellino, J. comput. Chem. 27: 995-1008, 2006.
% Equations are further modified by Byeongdu Lee.
% Programmed by Byeongdu Lee.
% 2007. 12. 11
% y = debyem_fast(q, x, y, z, atm, Fq);
% coord : [x, y, z] and should have 3 column
% atm : type of atm (1,2,3...) and should have the same number of element
% of x
% Fq : Fq(q, atmtype), thus numel(q) row and max(atmtype) column.
%

factor = 1;
q = q(:);
coord = [x(:), y(:), z(:)];

if nargin>4
    atmtype = unique(atm);
    t = numel(x)/numel(atm);
    if t > 1
        atm = repmat(atm, 1, t);
        atm = atm(:);
    end
else
    atmtype = 1;
end

if (numel(atmtype) == 1) || (nargin <= 5)
    ddl = distNzero(coord);
    [delta, w] = getdeltaw(max(q), factor);
    S = nofsample(ddl, delta, w);
    [ul, dl] = hist(ddl, S);
    y = rhs(q, 2*ul, dl, factor);
%    y = rhs_multistep(q, 2*ul, dl, factor);
    y = numel(coord)/3+y(:);
    return;
else
    I0 = zeros(size(q));
    indx = cell(numel(atmtype),1);
    for i=1:numel(atmtype)
        indx{i} = find(atm == atmtype(i));
        I0 = I0 + abs(Fq(:, atmtype(i))).^2*numel(indx{i});
    end
    
    y = zeros(size(q));
    
    for m=1:numel(atmtype)
        for n=m:numel(atmtype)
            m_co = coord(indx{m}, :);
            n_co = coord(indx{n}, :);
            ddl = distNzero2(m_co, n_co);
            
            [delta, w] = getdeltaw(max(q), factor);
            S = nofsample(ddl, delta, w);
            [ul, dl] = hist(ddl, S);
            yrhs = rhs(q, ul, dl, factor);
            y = y + Fq(:,atmtype(m)).*Fq(:,atmtype(n)).*yrhs;
        end
    end
    y = I0 + y;
end
end

function y = rhs(q, ul, dl, factor)
    delta = dl(end) - dl(end-1);
    w = delta*(factor/2);
    Cuq = Cq(q, w);
    y = zeros(size(q));
    for i=1:numel(q)
        y(i) = Cuq(i).*sum(ul.*sinc(q(i)*dl));
    end
end

function y = rhs_multistep(q, ul, dl, factor)
    Smax = numel(dl);
%    d_dl = dl(end) - dl(end-1);
%    w_dl = d_dl*(factor/2);
%    Cuq = Cq(q, w);
    y = zeros(size(q));
    for i=1:numel(q)
        [d_new, w_new] = getdeltaw(q(i), factor);
        S = nofsample(dl, d_new, w_new);
        if S<5
            S = 5;
        end
        pnt = floor(Smax/S);
%        delta = pnt*delta;
%        uul = ul*pnt;
        t = floor(Smax/pnt);
        newdl = sum(reshape(dl(1:t*pnt), t, pnt),2);
        newul = sum(reshape(ul(1:t*pnt), t, pnt),2);
        if t*pnt < Smax
            newdl = [newdl;sum(dl(t*pnt+1:end))];
            newul = [newul;sum(ul(t*pnt+1:end))];
        end
%        [uul, ddl] = hist(dl, S);     
        delta = newdl(end)-newdl(end-1);
        w = delta*(factor/2);
        y(i) = Cq(q(i), w).*sum(newul.*sinc(q(i)*newdl));
    end
end

function [S,D] = nofsample(dl, delta, w)   % for a box function of U(x)
    D = max(dl);
    S = floor((D+w/2)/delta);
end

function [delta, w] = getdeltaw(qmax, factor)
    % factor should be bigger than 1;
    if nargin ==1
        factor = 1;
    end
    if factor < 1;
        disp('factor should be bigger than 1')
    end
    deltamax = 1/qmax/2;
    delta = deltamax/factor;
    w = delta*(factor/2); % w should be smaller than 2*deltamax and bigger than delta, for a box function of U(x).
end

function x = Cq(q, w)   % C fuction for a box function: Eq. 19
    x = (pi*q*w)./(sin(pi*q*w));
end