function [sq, R, gm, ggr] = r2iq(q, x, y, z, Vbox, atm, Fq)
%function [Sq, r,pr] = r2iq(q, x, y, z, resolution, Vbox, atm, Fq)
% Calculate the interference function from coordinates and form factor of
% particles.
% assumption: the position and orientation of particles are not correlated.
% Fq is the mean Fq (orientation averaged)
% Byeongdu Lee.

[fqr, fqc] = size(Fq);
if numel(q) ~= fqr
    disp('the raw number of Fq is different from the number of q')
    return
end

N_type = unique(atm);
if numel(N_type) ~= fqc
    disp('the column number of Fq is different from the type of particles')
    return
end

%if nargin< 5
%    resolution = [];
%    atm = [];
%end
%if nargin < 6
%    atm = [];
%end

%if isempty(resolution)
%    resolution = 0.1/max(q);
%end

N_particle = numel(x);
np = N_particle/Vbox;
indmat = triu(ones(N_type));

r = cell(sum(sum(indmat)), 1);
nr = cell(sum(sum(indmat)), 1);
%coord = [x(:),y(:),z(:)];
x = x(:);
y = y(:);
z = z(:);
m = 0;

for i=1:N_type
    for j=i:N_type
        k = atm == i;
        c1 = [x(k), y(k), z(k)];
        if j~=i
            k = atm == j;
            c2 = [x(k), y(k), z(k)];
        else
            c2 = c1;
        end
        m = m+1;
        if j~=i
            [r{m}, nr{m}] = distNzero2c(c1, c2);
        else
            [r{m}, nr{m}] = distNzero2c(c1);
        end
    end
end
R = [];
for i=1:m
    R = [R(:);r{i}];
end
R = unique(R);

R(R>sqrt(Vbox)) = []; % for 2D
R(R>(Vbox)^(1/3)) = []; % for 2D

gr = zeros(numel(R), m);
%ggr = zeros(numel(q), numel(R));
gm = frsquare(R, sqrt(Vbox)); % for 2D.
gm = gm(:);

m=0;
for i=1:N_type
    for j=i:N_type
        m = m+1;
        [~, id1, id2] = intersect(R, r{m});
        %gr(id1, m) = nr{m}(id2)./gm(id2)./(np*4*pi*(r{m}(id2)).^2)+1;%gr =
        %nr/(np*4*pi*r^2) for 3D
        gr(id1, m) = nr{m}(id2)./gm(id2)./(2*pi*r{m}(id2))/np;%gr = nr/(np*2*pi*r) for 2D.
    end
end

sumFq = zeros(size(q));
n = zeros(N_type, 1);
for i = 1:N_type
    n(i) = numel(atm == i)/N_particle;
end

for i=1:N_type
    for j=1:N_type
        sumFq = sumFq + n(i)*n(j)*Fq(:,i).*Fq(:,j);
    end
end
size(sumFq)
sumFq = repmat(sumFq, [1, numel(R)]);
sumFqgr = zeros(size(sumFq));

for i=1:N_type
    for j=1:N_type
        ind = sum(sum(indmat(1:(i-1), :))) + sum(indmat(i, 1:j));
        sumFqgr = sumFqgr + n(i)*n(j)*(Fq(:,i).*Fq(:,j))*gr(:, ind)';
    end
end
ggr = sumFqgr./sumFq;

ggr = ggr/np;
sq = zeros(size(q));
for i=1:numel(q)
    sq(i) = pdf2sq(R, ggr(i,:), q(i), np);
end