function [W, D] = wishrnd(Sigma, df, D, n)
if nargin < 4
    n = 1;
end
if (nargin < 2)
  print_usage ();
end

if nargin < 3 || isempty(D)
  try
    D = chol(Sigma);
  catch
    error('wishrnd: Cholesky decomposition failed; Sigma probably not positive definite')
  end
end

p = size(D, 1);

if df < p
  df = floor(df); 
  df_isint = 1;
else 
 df_isint = (df == floor(df));
end

if ~df_isint
  [ii, jj] = ind2sub([p, p], 1:(p*p));
end

if n > 1
  W = nan(p, p, n);
end

for i = 1:n
  if df_isint
    Z = randn(df, p) * D;
    W(:, :, i) = Z'*Z;
  else
    Z = diag(sqrt(chi2rnd(df - (0:(p-1))))); 
    Z(ii > jj) = randn(p*(p-1)/2, 1);
    Z = D * Z;
    W(:, :, i) = Z*Z';
  end

  
end

end