function varargout = drawhelicalcylinder(param, varargin)
%% draw helix
Rh = param(1);
R = param(2);
ph = param(3);
N = param(4);

Nh = 100;
t = linspace(0, 2*pi, Nh);
t = N*t(:);

if Rh > 0
    v = ph/(2*pi*Rh);
    vnz = (1+v^2)^(-1/2);
    Rh_vnz = Rh/vnz;
    helixlength = 2*pi*Rh_vnz;
    rh = Rh*[cos(t), sin(t), v*t];
    nt = vnz*[-sin(t), cos(t), v*ones(length(t), 1)];
else
    vnz = 0;
    Rh_vnz = ph/(2*pi);
    helixlength = ph;
    v = ph/(2*pi);
    rh = [0*cos(t), 0*sin(t), v*t];
    nt = [0*t, 0*t, t./t];
end
n0 = [0, 0, 1];
nx = [1, 0, 0];
ny = [0, 1, 0];
Ncircle = 20;
m = linspace(0, 2*pi, Ncircle);m = m(:);%m(end) = [];
cir = R*[cos(m), sin(m), zeros(size(m))];
% Rotate, n0 to nt(1);
hx = zeros(length(t), Ncircle);
hy = zeros(length(t), Ncircle);
hz = zeros(length(t), Ncircle);
for i=1:length(t)
    nx = [rh(i, 1:2)/Rh, 0];
    nn = cross(nx, nt(i, :));
    R = [nx(:), nn(:), nt(i, :)'];
    invR = R';
    hc = cir*invR + rh(i, :);
    hx(i, :) = hc(:, 1)';
    hy(i, :) = hc(:, 2)';
    hz(i, :) = hc(:, 3)';
end
%% Draw the torus

% default set of options for drawing meshes
options = {'FaceColor', 'r', 'linestyle', 'none'};
for i=1:2:numel(varargin)
    switch lower(varargin{i})
        case 'facecolor'
            options{2} = varargin{i+1};
        case 'linestyle'
            options{4} = varargin{i+1};
        otherwise
            options{5} = varargin{i};
            options{6} = varargin{i+1};
    end
end

% draw the surface
hs = surf(hx, hy, hz, options{:});
axis image;
light
lighting gouraud;

%% Process output arguments

if nargout > 0
    varargout = {hs};
end
