function ret = equationofplane(varargin)
% ret = equationofplane(p1, p2, p3)
% ret = [A, B, C, D];
%
% usage 1:
% ret = equationofplane(p1, p2, p3)
%
% usage 2:
% ret = equationofplane(obj)
%   obj.vertices
%   obj.faces
%
% the standard equation of a plane in 3D is
% Ax + By + Cz + D = 0
% The sign of s = Ax + By + Cz + D determines which side the point (x,y,z) 
% lies with respect to the plane. If s > 0 then the point lies on the same 
% side as the normal (A,B,C). If s < 0 then it lies on the opposite side, 
% if s = 0 then the point (x,y,z) lies on the plane. 
if numel(varargin) == 1
    o = varargin{1};
    ret = zeros(size(o.faces, 1), 4);
    for i=1:size(o.faces, 1)
       p1 = o.vertices(o.faces(i,1), :);
       p2 = o.vertices(o.faces(i,2), :);
       p3 = o.vertices(o.faces(i,3), :);
       ret(i, :) = equationofplane(p1, p2, p3);
    end
    return
end

if numel(varargin) == 3
    p1 = varargin{1};
    p2 = varargin{2};
    p3 = varargin{3};
end

A = p1(2) *(p2(3) - p3(3)) + p2(2) *(p3(3) - p1(3)) + p3(2) *(p1(3) - p2(3));
B = p1(3) *(p2(1) - p3(1)) + p2(3) *(p3(1) - p1(1)) + p3(3) *(p1(1) - p2(1));
C = p1(1) *(p2(2) - p3(2)) + p2(1) *(p3(2) - p1(2)) + p3(1) *(p1(2) - p2(2));
D = -1*(p1(1) *(p2(2)* p3(3) - p3(2)* p2(3)) + p2(1) *(p3(2) *p1(3) - p1(2)...
    *p3(3)) + p3(1) *(p1(2) *p2(3) - p2(2) *p1(3)));
ret = [A, B, C, D];