function subunit = find_polyhedron_subunit(a, nface)
% function subunit = find_polyhedron_subunit(a, nface)
% find the subunit of a polyhedron
% a should be a object of a polyhedron
% example;
% o = polyhedra('truncatedcube');
% s = find_polyhedron_subunit(o);
% or
% s = find_polyhedron_subunit(o, 1);
%
% 2012, 12, 8
% B. Lee
subunit = [];
if iscell(a.faces)
    if nargin==1
        nf = cellfun(@numel, a.faces);
        [n, id] = unique(nf);
    else
        n=numel(nface);
        id = nface;
    end
    CM = mean(a.vertices);
    for i=1:numel(n)
        face = a.faces{id(i)};
        V = a.vertices(face, :,:);
        subunit{i} = findsub([V;CM], face);
    end
else
    if nargin == 1
        id = 1;
    else
        id = nface;
    end
    CM = mean(a.vertices);
    if numel(id) == 1
        subunit = findsub([a.vertices(a.faces(id,:),:,:);CM], a.faces(id,:));
    else
        for j=1:numel(id)
            face = a.faces(id(j),:);
            subunit{j} = findsub([a.vertices(face,:,:);CM], face);
        end
    end
end
end

function su = findsub(V, face)
    %CM = mean(V);
    nf = numel(face);
    if nf == 3
        newface = 1:3;
        for j=1:nf
            newface = [newface;j, j+1, nf+1];
        end
    else
        newface{1} = 1:nf;
        for j=1:nf
            newface{j+1} = [j, j+1, nf+1];
        end
    end

    su.vertices = V;
    su.faces = newface;

end