function F = findfaces(vert)
% function F = findfaces(vert)
% When vertices of a polyhedron is given, [x0,y0,z0; ... ;xn, yn, zn],
% This function finds a faces using convhull.
%
% Byeongdu Lee
% Dec. 5. 2012
% 6/15/2013, modified.

% triangulate..
% convhull will generate triangles that is at outer surface of polyhedron.
tri = convhull(vert);
v = zeros(size(tri, 1), 3);

% the following calculate the orthonormal vector to each triangle.
for i=1:size(tri,1)
    p = vert(tri(i,:), :);
    t=planeNormal([p(1,:) p(1,:)-p(2,:) p(1,:)-p(3,:)]); 
    v(i, :) = t/norm(t);
end

% Now, the code below will check which triangle's normal vectors are identical
% If normal vector of two triangles are identical, then all the points
% associated to the two triangles are on a same plane.
%v = round(v*1000)/1000;
%nv = reducehkl(v);
[nv, pfaces, index2] = unique_m(v, 1E5); % nv: unique 

F = cell(size(nv,1),1);
for i=1:size(nv, 1)
    % find vertices associated to the same plane.
    if iscell(pfaces)
        F{i} = unique(tri(pfaces{i},:));
    else
        F{i} = unique(tri(index2 == i, :));
    end
    
    % sort the vertices in the sequence of connecting edges.
    pF = vert(F{i}, :);
    seq = planeconstruct(pF);
%     sF = pF(1,:);
%     seq = [1];
%     pF(1,:) = [inf, inf, inf];
%     for k=1:numel(F{i})-1
%         % check the nearest point and pick the one as the next point.
%         d = sqrt((pF(:,1)-sF(1)).^2+(pF(:,2)-sF(2)).^2+(pF(:,3)-sF(3)).^2);
%         [~, s] = sort(d);
%         seq = [seq, s(1)];
%         sF = pF(s(1), :);
%         pF(s(1),:) = [inf, inf, inf];
%     end
    F{i} = F{i}(seq);
end