function rotateobj_around_vector(h, angle, vector)
%rotateobj_around_vector Rotate objects about specified axis.
%   ROTATEOBJ(H, angle, vector) rotates the objects with handles H
%
%   Took the code from ROTATE.m and modified slightly.
%   Byeongdu Lee
%   12/30/2014
%
%   See also ROTATE, SPH2CART, CART2SPH.

%   Copyright 1984-2012 The MathWorks, Inc.


% find unit vector for axis of rotation
origin = [0, 0, 0];
rot = rotate_around_vector(vector, angle);
for i=1:numel(h),
  t = h(i).Type;
  skip = 0;
  if strcmp(t,'surface') || strcmp(t,'line') || strcmp(t,'patch')
    
    % If patch, rotate vertices  
    if strcmp(t,'patch')
       verts = h(i).Vertices;
       x = verts(:,1); y = verts(:,2); 
       if size(verts,2)>2
          z = verts(:,3);
       else
          z = [];
       end
       
    % If surface or line, rotate {x,y,z}data   
    else
       x = h(i).XData;
       y = h(i).YData;
       z = h(i).ZData;
    end
    
    if isempty(z)
       z = -origin(3)*ones(size(y));
    end
    [m,n] = size(z);
    if numel(x) < m*n
      [x,y] = meshgrid(x,y);
    end
  elseif strcmp(t,'text')
    p = h(i).Position;
    x = p(1); y = p(2); z = p(3);
  elseif strcmp(t,'image')
    x = h(i).XData;
    y = h(i).YData;
    z = zeros(size(x));
  else
    skip = 1;
  end
  
  if ~skip,
    [m,n] = size(x);
    newxyz = [x(:)-origin(1), y(:)-origin(2), z(:)-origin(3)];
    newxyz = newxyz*rot;
    newx = origin(1) + reshape(newxyz(:,1),m,n);
    newy = origin(2) + reshape(newxyz(:,2),m,n);
    newz = origin(3) + reshape(newxyz(:,3),m,n);

    if strcmp(t,'surface') || strcmp(t,'line')
      h(i).XData = newx;
      h(i).YData = newy;
      h(i).ZData = newz;
    elseif strcmp(t,'patch')
      h(i).Vertices = [newx,newy,newz]; 
    elseif strcmp(t,'text')
      h(i).Position = [newx newy newz];
    elseif strcmp(t,'image')
      h(i).XData = newx;
      h(i).YData = newy;
    end
    drawnow
  end
end
