classdef PDB < molecule & atom
    properties (SetAccess = public)
        outfile
        recordName
        atomNum
        atomName
        altLoc
        resName
        chainID
        resNum
% X, Y, Z are defined in molecule        
%        X
%        Y
%        Z
        occupancy
        betaFactor
        element
        charge
        BIOMT
    end
    methods
        function obj = PDB(filename)
            % PDB(filename), PDB({'Ca', 'B', 'O'}), or PDB(pdb)
            obj.outfile = filename;
            if ischar(filename)
                obj.loadpdb(filename)
            elseif iscell(filename)
                obj.element = filename;
            elseif isstruct(filename)
                obj.element = filename.element;
                obj.X = filename.x;
                obj.Y = filename.y;
                obj.Z = filename.z;
            end
            obj.lowerElementName;
            obj.updatemolecule;
        end
        function obj = updatemolecule(obj)
            obj.Element = obj.element;
            obj.getAtomicN;
            obj.updatefromAtomicN;
        end
        function loadpdb(obj, filename)
            a = pdb2mat(filename);
            fn = fieldnames(a);
            for i=1:numel(fn)
                obj.(fn{i}) = a.(fn{i});
            end
        end
        function lowerElementName(obj)
            S = obj.element;
            t = cellfun(@(x) numel(x), S);
            ind = find(t>1);
            for i=1:numel(ind)
                indtobereplaced = ind(i);
                S{indtobereplaced} = ...
                    strcat(upper(S{indtobereplaced}(1)),...
                    lower(S{indtobereplaced}(2:end)));
            end
            obj.element = S;
        end
        function operateBIOMT(obj)
            pos = [obj.X(:), obj.Y(:), obj.Z(:)];
            atm = obj.Atom;
            if ~isprop(obj, 'BIOMT')
                fprintf('There is no ''BIOMT'' property to operate.\n');
                return
            end
            if size(atm, 1) == numel(obj.BIOMT)
                fprintf('Already BIOMT has been applied. No further action will be done.\n');
                return
            end
                
            a = [];p = [];
            for i=1:numel(obj.BIOMT)
                a = [a;atm]; 
                p = [p;pos*obj.BIOMT{i}']; 
            end
            asize = size(a);
            obj.X = reshape(p(:,1), asize);
            obj.Y = reshape(p(:,2), asize);
            obj.Z = reshape(p(:,3), asize);
            obj.Atom = a;
        end
    end
end