function model = PDB2box(pdb, resolution)
% Use BOXmodel instead.
% pxxx = PDB('xxx.pdb');
% md = BOXmodel(pxxx);
% Calculate Fq 
%   md.qv = [qx, qy, qz];
%   md.Fqcal
% Calculate Fq with FFT
%   md.Fqfft
%   md.Pqorientavg
% Calculate Pq with debye function
%   md.q = 1:10;
%   md.debyePq
%
%  PDB with properties:
% 
%          outfile: 'Wah_WholePC_from_2XYY.pdb'
%       recordName: {1x174720 cell}
%          atomNum: [1x174720 double]
%         atomName: {1x174720 cell}
%           altLoc: {1x174720 cell}
%          resName: {1x174720 cell}
%          chainID: {1x174720 cell}
%           resNum: [1x174720 double]
%                X: [1x174720 double]
%                Y: [1x174720 double]
%                Z: [1x174720 double]
%        occupancy: [1x174720 double]
%       betaFactor: [1x174720 double]
%          element: {1x174720 cell}
%           charge: {1x174720 cell}
%             Atom: [1x174720 double]
%          Formula: 'C174720'
%          Element: {1x174720 cell}
%               Mw: 2.0986e+06
%     AtomicNumber: [1x174720 double]
%      ElementName: []
%           Symbol: []
%           Radius: [1x174720 double]
%             Mass: []
%
% to draw the object
% p = patch(model.object);
% p.FaceColor = 'r';
% p.EdgeColor = 'none';
% p.FaceAlpha = 0.5;
% daspect([1,1,1])
% view(3);
% axis tight
% camlight
% lighting gouraud

xmin = min(pdb.X);
xmax = max(pdb.X);
ymin = min(pdb.Y);
ymax = max(pdb.Y);
zmin = min(pdb.Z);
zmax = max(pdb.Z);
m = min([xmin, ymin, zmin]); m = floor(m);
n = max([xmax, ymax, zmax]); n = ceil(n);
x = m:resolution:n;

boxvolume = resolution^3;
[a, ~,cpb] = histcn([pdb.X(:),pdb.Y(:),pdb.Z(:)], x, x, x, 'AccumData', pdb.Atom);
% cpb = center positions of the small resolution boxes.
% a : atomic density
model.boxvolume = boxvolume;
model.edensity = a;
model.cpb = cpb;
t = isosurface(cpb{1}, cpb{2},cpb{3}, a);
model.object = t;
model.resolution = resolution;
%model.Fq = fftn(model.edensity);
Y = model.edensity;

qnum = numel(x)*3;
if qnum > 300;
    qnum = 300;
end
q = linspace(-1/max(x)/2, 1/max(x)/2, qnum);
q = q/resolution*max(x)*(2*pi);
for p = 1:length(size(model.edensity))
    Y = fft(Y,qnum,p);
end
model.Fq = Y;
model.Fq = fftshift(model.Fq, 1);
model.Fq = fftshift(model.Fq, 2);
model.Fq = fftshift(model.Fq, 3);
model.Pq = abs(model.Fq).^2;
model.q = q(:);

% orientational average of Pq
[qx, qy, qz] = meshgrid(q, q, q);
qv = sqrt(qx.^2+qy.^2+qz.^2);
qv = qv(:);
Pq = model.Pq(:);
q1d = linspace(0, sqrt(2)*max(q), qnum*2);
qarray = q1d-q1d(1);
[h, bin] = histc(qv, qarray);
q = qarray(1:end-1) + diff(qarray);
t = bin == 0;
bin(t) = [];
Pq(t) = [];

c=accumarray(bin,Pq);
%c(end) = [];
if numel(h)>numel(c)
    h(end) = [];
end
Iqv = c./max(1,h);
c = Iqv;
if numel(c) > numel(q)
    c(end) = [];
end
model.Pq1d = [q(:), c(:)];

