classdef aminoacid
    properties
        amino = {'Alanine', 'ALA',	'A',	1, 89.09, [3, 7, 1, 2, 0];
        'Arginine', 'ARG', 	'R',	2, 174.2, [6, 14, 4, 2, 0];
        'Asparagine', 'ASN', 	'N',	3, 132.12, [4,8,2,3,0];
        'Aspartic acid', 'ASP' 	'D',	4, 133.10, [4, 7, 1, 4, 0];
        'Cysteine', 'CYS', 	'C',	5, 121.16, [3,7,1,2,1];
        'Glutamine', 'GLN', 	'Q',	6, 146.14,[5,10,2,3,0];
        'Glutamic acid', 'GLU', 	'E',	7,147.13,[5,9,1,4,0];
        'Glycine',  'GLY',	'G',	8,75.07,[2,5,1,2,0];
        'Histidine', 'HIS',	'H',	9, 155.15, [6,9,3,2,0];
        'Isoleucine', 'ILE',	'I',	10, 131.17, [6,13,1,2,0];
        'Leucine', 'LEU', 	'L',	11, 131.17,[6,13,1,2,0];
        'Lysine', 'LYS',	'K',	12,146.19,[6,14,2,2,0];
        'Methionine', 'MET',	'M',	13,149.21, [5,11,1,2,1];
        'Phenylalanine', 'PHE', 	'F',	14, 165.19, [9,11,1,2,0];
        'Proline', 'PRO',	'P',	15, 115.13, [5,9,1,2,0];
        'Serine', 'SER',	'S',	16, 105.09, [3,7,1,3,0];
        'Threonine', 'THR',	'T',	17, 119.12, [4,9,1,3,0];
        'Tryptophan', 'TRP', 	'W',	18, 204.23, [11,12,2,2,0];
        'Tyrosine', 'TYR', 	'Y',	19, 181.19, [9,11,1,3,0];
        'Valine', 'VAL', 	'V',	20, 117.15, [5,11,1,2,0]};
    end
% amino acid, 3 letter, 1 letter, integer, Mw(g/mol), [C H N O S]
    methods
        function obj = aminoacid(varargin)
            % Calculate number of electrons.
            Ncol = size(obj.amino, 2);
            for i=1:size(obj.amino, 1)
                obj.amino{i, Ncol+1} = obj.amino{i, Ncol}*[6, 1, 7, 8, 16]';
            end
        end
        function index = findindex(obj, Lett)
            switch numel(Lett)
                case 3
            % 3 Letter
                    t = strfind([obj.amino{:,2}], Lett); index = (t-1)/3+1;
                case 1
                    t = strfind([obj.amino{:,3}], Lett); index = t;
                otherwise
                    index = 0;
            end
        end
        function Nelectron = Ne(obj, Lett)
            index = findindex(obj, Lett);
            if index == 0
                Nelectron = 0;
                return
            end
            Nelectron = obj.amino{index, 7};
        end
        function M = Mw(obj, Lett)
            index = findindex(obj, Lett);
            if index == 0
                M = 0;
                return
            end
            M = obj.amino{index, 5};
        end
        function C = CF(obj, Lett)
            index = findindex(obj, Lett);
            if index == 0
                C = [];
                return
            end
            M = obj.amino{index, 6};
            if ~M(end)
                C = sprintf('C%iH%iN%iO%i', M(1:4));
            else
                C = sprintf('C%iH%iN%iO%iS%i', M);
            end
        end
    end
end            