function fq = fq_pdb(qx, qy, qz, x, y, z, atm, xyangle, yzangle)
% Calculate the SAXS form factor from atomic coordinates and type of
% atom. Instead of atomic scattering factor, atomic number is used.
%
% Byeongdu Lee

if nargin <= 7
    xyangle = 0;
    yzangle = 0;
elseif nargin <=8
    yzangle = 0;
end

xt = qx;
yt = qy;

qx = cos(xyangle)*xt + sin(xyangle)*yt;
qy = -sin(xyangle)*xt + cos(xyangle)*yt;

% rotate pyramid 45degree in yz plane.
yt = qy;
zt = qz;

qy = cos(yzangle)*yt + sin(yzangle)*zt;
qz = -sin(yzangle)*yt + cos(yzangle)*zt;
        % ===============================================
j = sqrt(-1);
[nx, ny] = size(qx);
if nx>ny
    Q = [qx, qy, qz];
else
    Q = [qx', qy', qz'];
end
q = sqrt(Q(:,1).^2+Q(:,2).^2+Q(:,3).^2);
[nx, ny] = size(x);
if nx>ny
    R = [x'; y'; z'];
else
    R = [x;y;z];
end
atm = atm(:)';
if max(q) < 1
    atm = repmat(atm, size(Q, 1),1);
else
    [allatm, ~, c] = unique(atm);
    atmFq = zeros(numel(q), numel(allatm));
    for i=1:numel(allatm);
        atmFq(:,i) = atomicscatteringfactor(q, i);
    end
    atm = atmFq(:,c);
end
%fq = zeros(size(qx));      
fq = sum(atm.*exp(-j*Q*R),2);
%for i=1:numel(x);
%    fq = fq+atm(i)*exp(-j*(qx*x(i)+qy*y(i)+qz*z(i)));
%end