classdef molecule2 < handle
    properties (Hidden)
    AN_
    ElementName_
    Sym_
    Radius_
    Color_
    RAMass_
    end
    properties (SetAccess = public)
        Atom
        Formula
        Element
        Mw
    end
    methods
        function obj = molecule2(varargin)
            % atomic number : [ 1, 5, 10]....
            % copy the Hidden properties from the class atom.
            mco = ?molecule2;
            plist = mco.PropertyList;
            a = atom;
            ishidden = [plist.Hidden];
            fn = {plist.Name};
            fn = fn(ishidden);
            for i=1:numel(fn)
                obj.(fn{i}) = a.(fn{i});
            end
            
            if isempty(varargin)
                obj.Atom = [];
                obj.Element = [];
                return;
            end
            if isnumeric(varargin{1})
                obj.Atom = varargin{1};
                obj.getElm;
            elseif ischar(varargin{1})
                if numel(varargin) == 1
                    obj.Formula = varargin;
                    obj.parse_CF;
                    return;
                end
                obj.Element = varargin;
                obj.getAtomicN;
            elseif iscell(varargin{1})
                obj.Element = varargin{1};
                obj.getAtomicN;
            end
            obj.getMw;
            obj.getCF;
        end
        function getAtomicN(obj)
            obj.Atom = S2AN(obj, obj.Element);
        end
        function getElm(obj)
            obj.Element = AN2S(obj, obj.Atom);
        end
        function AN = S2AN(obj, S)
            if isempty(S)
                error('Does not contain any symbol\n');
            end
            AN = zeros(size(S));
            for i=1:numel(S)
                try
                    AN(i) = findcellstr(obj.Sym_, S(i));
                catch
                    errstr = sprintf('The element name %s is not found in the molecule class.\n', S{i});
                    error(errstr);
                end
            end
        end
        function S = AN2S(obj, AN)
            if isempty(AN)
                error('Does not contain any symbol\n');
            end
            S = obj.Sym_(AN);
        end
        function getMw(obj)
            if isempty(obj.Atom)
                obj.getAtomicN;
            end
            obj.Mw = sum(obj.RAMass_(obj.Atom));
        end
        function obj = getCF(obj)
            if isempty(obj.Atom)
                obj.getAtomicN;
            end
            [a, ~, c] = unique(obj.Atom);
            N = hist(c, numel(a));
            str = '';
            for i=1:numel(N)
                if N(i) == 1
                    obj.Sym_(a(i))
                    str = sprintf('%s%s', str, obj.Sym_{a(i)});
                else
                    str = sprintf('%s%s%i', str, obj.Sym_{a(i)}, N(i));
                end
            end
            obj.Formula = str;
        end
        function obj = parse_CF(obj)
            a = parse_formula(obj.Formula);
            fn = fieldnames(a);
            %obj.Element = fn;
            N = cell2mat(struct2cell(a));
            AtN = cell(1, sum(N));
            k = 1;
            for i=1:numel(N)
                AtN(k:(k+N(i)-1)) = fn(i);
                k = k + N(i);
            end
            obj.Element = AtN;
            obj.getAtomicN;
            obj.getMw;
        end
        function updatefromAtomicN(obj)
            obj.getElm;
            obj.getMw;
            obj.getCF;
            obj.getMw;
        end
        function draw(obj)
            figure;light;hold on;
            light('Position',[-1 -1 -2]);
            [x,y,z] = sphere(10);
            if ~isprop(obj, 'X')
                error('No atomic coordinates.\n')
            end
            for i=1:numel(obj.X);
                r = obj.Radius_(obj.Atom(i));
                color = obj.Color_(obj.Atom(i), :);hold on;
                surface('XData',obj.X(i) + r*x,'YData',obj.Y(i) + r*y,...
                    'ZData',obj.Z(i) + r*z,'FaceColor',color,...
                    'EdgeColor','none','FaceLighting','gouraud')
            end
            axis image;
        end
    end
end
