function varargout = pdbread3(filename)
% function [atom, hetatom, matrix, Tmat] = pdbread3(filename)
% atom and hetatom have following fields.
% .atom
% .x
% .y
% .z
% 
% function [atom, x, y, z, matrix, Tmat] = pdbread3(filename)
% matlab version..

amino = aminoacid;
isfullatomformat = 0;

model = {};
Nmodel = 0;
atoms = [];
atoms.element = {};
atoms.x = [];
atoms.y = [];
atoms.z = [];
hetatom.element = {};
hetatom.x = [];
hetatom.y = [];
hetatom.z = [];
x = [];
y = [];
z = [];
cell = [];
ismatrix = 0;
fid = fopen(filename, 'rt');
if (fid<0)
        error(['Couldn''t find ' file]);
end
matrix = zeros(3, 3, 1);
Tmat = zeros(3, 1);
line = fgetl(fid);
isfullatomformat = 0;

while line~=-1
    head = strtok(line, ' ');
    NN = -1;
    if numel(head) > 6
        head(7:end) = [];
    end
%    if strcmp(head,'ATOM') || strcmp(head, 'HETATM')

    if strcmp(head, 'CRYST1')
       cell.a = str2double(line(7:15)); 
       cell.b = str2double(line(16:24)); 
       cell.c = str2double(line(25:33)); 
       cell.alpha = str2double(line(34:40)); 
       cell.beta = str2double(line(41:47)); 
       cell.gamma = str2double(line(48:54)); 
       cell.spacegroup = strtrim(line(56:66));
       cell.Zvalue = str2num(line(67:70));
    end
    if strcmp(head, 'MODEL')
        str = strread(line, '%s');
        Nmodel = str2num(strtrim(line(13:16)));
    end
    if strcmp(head, 'ENDMDL')
        model{Nmodel}.atom = atoms;
        model{Nmodel}.hetatom = hetatom;
        atoms = [];
        atoms.atom = [];
        atoms.x = [];
        atoms.y = [];
        atoms.z = [];
        hetatom.atom = [];
        hetatom.x = [];
        hetatom.y = [];
        hetatom.z = [];    
    end
        
    if strcmp(head,'ATOM') || strcmp(head, 'HETATM')
    str = textscan(line, '%s');
    a{1} = line(1:6); % Record Name
    a{2} = str2num(line(7:11)); % Atom Serial number
    a{3} = strtrim(line(13:16)); % Atom name
    a{4} = strtrim(line(17)); % Alternate location indicator.
    a{5} = strtrim(line(18:20)); % Residue name
    a{6} = strtrim(line(22)); % chain identifier
    a{7} = str2num(line(23:26)); % Residue sequence number
    a{8} = line(27); % %Code for insertion of residues
    a{9} = str2double(line(31:38));% Orthogonal coordinates for X in Angstroms.
    a{10} = str2double(line(39:46));% Orthogonal coordinates for Y in Angstroms.
    a{11} = str2double(line(47:54)); % Orthogonal coordinates for Z in Angstroms.
    a{12} = str2double(line(55:60)); % Occupancy.
    a{13} = str2double(line(61:66)); % Temperature factor (Default = 0.0)
    if length(line)>66
    a{14} = strtrim(line(73:76)); % Segment identifier, left-justfied.
    end
    NN = [];
    if length(line)>77
        a{15} = strtrim(line(77:78)); % Element Symbol, right-justfied.
        if ~isempty(a{15})
            NN = str2num(a{15}(1));
        end
    end
    if length(line)>79
        a{16} = strtrim(line(79:80)); % Charge on the atom
        isfullatomformat = 1;
    end
%	switch str{end}
    %NN = [fprintf('%i', a{15}(1))];
%    if ((NN >=48) & (NN <=57)); % if numeric
    if isfullatomformat
        if isempty(NN)
            atomsymbol = a{15};
        end
        if isempty(atomsymbol)
            atomsymbol = a{3};
            asciicode = double(atomsymbol);
            t = asciicode<=64;
            atomsymbol(t) = [];
            if numel(atomsymbol)>1
                asciicode = double(atomsymbol(2));
                if asciicode < 91
                    atomsymbol = atomsymbol(1);
                end
            end
        end
    else
        if isempty(NN)
            atomsymbol = a{15};
        end
        if isempty(atomsymbol)
            atomsymbol = a{3};
        end
    end
    xs = a{9};
    ys = a{10};
    zs = a{11};
    if strcmp(head,'ATOM')
        %atom.atom = [atom.atom;atomn];
        atoms.element = [atoms.element;atomsymbol];
        atoms.x = [atoms.x;xs];
        atoms.y = [atoms.y;ys];
        atoms.z = [atoms.z;zs];
    end
        
    if strcmp(head, 'HETATM')
        %hetatom.atom = [hetatom.atom;atomn];
        hetatom.element = [hetatom.element;atomsymbol];        
        hetatom.x = [hetatom.x;xs];
        hetatom.y = [hetatom.y;ys];
        hetatom.z = [hetatom.z;zs];
    end
    
    %str2double(str{7}), str{end}
   elseif strcmp(head, 'MODEL')
%	[a, num] = sscanf(line, '%s %i', 'C');
%	num
%	fflush(1);
   elseif strcmp(head, 'MTRIX1') || strcmp(head, 'MTRIX2') || strcmp(head, 'MTRIX3')
%	[a, b, xn, yn, zn, tr] = sscanf(line, '%s %i %f %f %f %f', 'C'); % octave
%	matrix(1, :, b) = [xn, yn, zn];
%	Tmat(1, b) = tr;
        ismatrix = 1;
        str = strread(line, '%s'); % octave
        matrix(str2double(str{1}(end)), :, str2double(str{2})) = [str2double(str{3}), str2double(str{4}), str2double(str{5})];
        Tmat(str2double(str{1}(end)), str2double(str{2})) = str2double(str{6});
   end 
        line=fgetl(fid);
end
fclose(fid);

if ismatrix
    [sx, sy, sz] = size(matrix);
    subset_atom = [x';y';z'];
    numofatom = length(x);
    if numofatom ~= 0;
        for i=1:sz
            t = matrix(:,:,i)*subset_atom + repmat(Tmat(:, i), 1, numofatom);
        %    if i>1
                x = [x, t(1,:)'];
                y = [y, t(2,:)'];
                z = [z, t(3,:)'];
        %    else
        %	x = t(1, :)';
        %	y = t(2, :)';
        %	z = t(3, :)';
         %   end
        end
    end
end
if numel(model) == 0
    if ~isempty(atoms.x)
        atoms = PDB(atoms);
    end
    if ~isempty(hetatom.x)
        hetatom = PDB(hetatom);
    end
    varargout = {atoms, hetatom, matrix, Tmat, cell};
else
    varargout = {model, matrix, Tmat, cell};
end

