function [Pq, Fqm] = pq_pdb(q, pdb, dim)
% [Pq, Fqm] = pq_pdb(q, pdb, dim)
% calculate SAXS P(q) from pdb and <Fq>_orient
% Look also pdb_Fq.m

% Byeongdu Lee
if nargin < 3
    dim = 3; % 3D random orientation
end

if isa(pdb, 'PDB');
    atom = pdb.Atom;
    x = pdb.X;
    y = pdb.Y;
    z = pdb.Z;
else
    atom = pdb.atom;
    x = pdb.x;
    y = pdb.y;
    z = pdb.z;
end

atmN = unique(atom);
maxAtmNumber = max(atmN);
atmFq = zeros(1, maxAtmNumber);

if max(q)<1
    atmFq(atmN) = atmN;
else
    atmFq = [];
end

if isempty(atmFq)
    qv = q;
    if size(q, 2) ==2
        qv = sqrt(q(:,1).^2+q(:,2).^2);
    elseif size(q, 2) == 3
        qv = sqrt(q(:,1).^2+q(:,2).^2+q(:,3).^2);
    end
    atmFq = zeros(numel(qv), maxAtmNumber);
    for i=1:numel(atmN);
        atmFq(:,atmN(i)) = atomicscatteringfactor(qv, atmN(i));
    end
end

if numel(x) < 60000
    if nargout == 1
        switch dim
            case 3
                Pq = debyem(q, x, y, z, atom, atmFq);
            case 2
                Pq = debye2d(q, x, y, z, atom, atmFq);
        end
        return
    end
end

%N = 2^5;
N = 2^7;
N2 = N^2;
MAXpntat1compute = N2/10; % This number provides the fast calculation.
[Qx, Qy, Qz] = spiral_on_sphere(1, N2);

%if N^2*nrp*numq > MAXpntat1compute
if size(q,1)==1 && size(q,2)>1
    q = q(:);
end
QQ = q;
numq = numel(q(:,1));

qseg = round(N2/MAXpntat1compute);
NumQseg = round(N2/qseg);
% qseg = round(N2/NumQseg);

Pq = zeros(size(q));
Fqm = zeros(size(q));
tic
for seg=1:numq
    q = QQ(seg);
    qx = q*Qx';
    qy = q*Qy';
    qz = q*Qz';
    Fq = zeros(1, N2);
    for qxseg = 1:qseg
        if qxseg < qseg
            ind = (qxseg-1)*NumQseg+1:NumQseg*qxseg;
        else
            ind = (qxseg-1)*NumQseg+1:N2;
        end
        qxs = qx(ind);
        qys = qy(ind);
        qzs = qz(ind);
        
%    sizeqx = size(qx);
        qxs = qxs(:);
        qys = qys(:);
        qzs = qzs(:);
        ff = fq_pdb(qxs, qys, qzs, x, y, z, atom);
        Fq(ind) = ff(:)';
        Iqt = mean(abs(Fq).^2);
        if nargout ==2
            Fqt = mean(Fq);
        end
        fprintf('Running seg %i/%i of %i/%i.\n', qxseg, qseg, seg, numq)
    end
    Pq(seg) = Iqt;
    if nargout == 2
        Fqm(seg) = Fqt;
    end
end
toc    

