function voigtfit2(varargin)
  fitfunctionname = 'multivoigt';
  figH = initFigure;
  handles = guihandles(figH);

    function figH = initFigure
        try
            bestP = evalin('base', 'bestP');
        catch
             Nf = 1;
                for i=1:Nf
                    bestP.(['amp', num2str(i)]) = 1;
                    bestP.(['center', num2str(i)]) = i;
                    bestP.(['sigg', num2str(i)]) = 0.002;
                    bestP.(['sigl', num2str(i)]) = 0.002;
                end
                bestP.poly1 = 1E-5;
                bestP.poly2 = -2;
                bestP.poly3 = -1;
                bestP.poly4 = 0.001;
        end
pnames = fieldnames(bestP);
posScreen   = get(0,'screenSize');
hFigWidth   = 600;
hFigHeight  = numel(pnames)*20 + 100;
hFigPos     = [...
    posScreen(3)/2-hFigWidth/2,...
    posScreen(4)/2-hFigHeight/2,...
    hFigWidth,hFigHeight];

figH = figure(...
  'position'                    , hFigPos,...
  'visible'                       , 'on', ...
  'units'                         , 'pixel', ...
  'busyaction'                    , 'queue', ...
  'doublebuffer'                  , 'on', ...
  'handlevisibility'              , 'callback', ...
  'IntegerHandle'                 , 'off',...
  'interruptible'                 , 'on', ...
  'menubar'                       , 'none', ...
  'numbertitle'                   , 'off', ...
  'resize'                        , 'off', ...
  'name'                        , ['Data fit for ', fitfunctionname], ...
  'tag'                           , 'BLee_Fit', ...
  'toolbar'                       , 'none', ...
  'defaultaxesunits'              , 'pixels', ...
  'defaulttextfontunits'          , 'pixels', ...
  'defaulttextfontname'           , 'Verdana', ...
  'defaulttextfontsize'           , 12, ...
  'defaultuicontrolunits'         , 'pixels', ...
  'defaultuicontrolfontunits'     , 'pixels', ...
  'defaultuicontrolfontsize'      , 10, ...
  'defaultuicontrolfontname'      , 'Verdana', ...
  'defaultuicontrolinterruptible' , 'off');

% Data loading and selection control
uph = uipanel(...
  'units'                     , 'pixels', ...
  'parent'                    , figH, ...
  'bordertype'                , 'beveledin', ...
  'tag'                       , 'versionPanel');

uicontrol(...
  'style'                     , 'pushbutton', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , uph, ...
  'string'                  ,   'Load data',...
  'callback'                , @pb_loaddata_Callback,...
  'position'                  ,[10, hFigHeight-50,100,25], ...
  'tag'                       , 'pb_loaddata');
uicontrol(...
  'style'                     , 'pushbutton', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , uph, ...
  'string'                  ,   'Fit region read from cursors',...
  'callback'                , @pushbutton_qselect_Callback,...
  'position'                  ,[10, hFigHeight-130,200,25], ...
  'tag'                       , 'pushbutton_qselect');

uicontrol(...
  'style'                     , 'pushbutton', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , uph, ...
  'string'                  ,   '<',...
  'callback'                , @pb_LL_Callback,...
  'position'                  ,[180, hFigHeight-50-160,15,20], ...
  'tag'                       , 'pb_LL');
uicontrol(...
  'style'                     , 'pushbutton', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , uph, ...
  'string'                  ,   '>',...
  'callback'                , @pb_LU_Callback,...
  'position'                  ,[195, hFigHeight-50-160,15,20], ...
  'tag'                       , 'pb_LU');
uicontrol(...
  'style'                     , 'pushbutton', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , uph, ...
  'string'                  ,   '<',...
  'callback'                , @pb_RL_Callback,...
  'position'                  ,[180, hFigHeight-50-180,15,20], ...
  'tag'                       , 'pb_RL');
uicontrol(...
  'style'                     , 'pushbutton', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , uph, ...
  'string'                  ,   '>',...
  'callback'                , @pb_RU_Callback,...
  'position'                  ,[195, hFigHeight-50-180,15,20], ...
  'tag'                       , 'pb_RU');

uicontrol(...
  'style'                     , 'edit', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , uph, ...
  'string'                  ,   '',...
  'callback'                , @edit_Figure_Callback,...
  'position'                  ,[100, hFigHeight-50-50,100,25], ...
  'tag'                       , 'edit_Figure');
uicontrol(...
  'style'                     , 'edit', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , uph, ...
  'string'                  ,   '',...
  'callback'                , @edit_Lq_Callback,...
  'position'                  ,[100, hFigHeight-50-110,80,20], ...
  'tag'                       , 'edit_Lq');
uicontrol(...
  'style'                     , 'edit', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , uph, ...
  'string'                  ,   '',...
  'callback'                , @edit_Rq_Callback,...
  'position'                  ,[100, hFigHeight-50-130,80,20], ...
  'tag'                       , 'edit_Rq');
uicontrol(...
  'style'                     , 'edit', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , uph, ...
  'string'                  ,   '',...
  'callback'                , @edit_Lindex_Callback,...
  'position'                  ,[100, hFigHeight-50-160,80,20], ...
  'tag'                       , 'edit_Lindex');
uicontrol(...
  'style'                     , 'edit', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , uph, ...
  'string'                  ,   '',...
  'callback'                , @edit_Rindex_Callback,...
  'position'                  ,[100, hFigHeight-50-180,80,20], ...
  'tag'                       , 'edit_Rindex');

uicontrol(...
  'style'                     , 'text', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , uph, ...
  'string'                  ,   'Pick Figure',...
  'position'                  ,[10, hFigHeight-50-50,80,25], ...
  'tag'                       , 'text0');
uicontrol(...
  'style'                     , 'text', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , uph, ...
  'string'                  ,   'q min',...
  'position'                  ,[50, hFigHeight-50-110,50,20], ...
  'tag'                       , 'text1');
uicontrol(...
  'style'                     , 'text', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , uph, ...
  'string'                  ,   'q max',...
  'position'                  ,[50, hFigHeight-50-130,50,20], ...
  'tag'                       , 'text2');
uicontrol(...
  'style'                     , 'text', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , uph, ...
  'string'                  ,   'Left index',...
  'position'                  ,[20, hFigHeight-50-160,80,20], ...
  'tag'                       , 'text3');
uicontrol(...
  'style'                     , 'text', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , uph, ...
  'string'                  ,   'Right index',...
  'position'                  ,[20, hFigHeight-50-180,80,20], ...
  'tag'                       , 'text4');



% Fitting control
baseHpos = 200; % horizontal position where the controls for parameters start.
baseVpos = 70; % vertical position where the controls for parameters start.
% baseVpos is essentially the space that the following 5 controls take.
Fitbuttonwidth = 150;
uicontrol(...
  'style'                     , 'edit', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'right', ...
  'parent'                    , figH, ...
  'position'                  ,[baseHpos, 20,Fitbuttonwidth,25], ...
  'tag'                       , 'edit_savefilename');

uicontrol(...
  'style'                     , 'pushbutton', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , figH, ...
  'string'                  ,   'Load Saved Result',...
  'callback'                , @pushbutton_Load_Callback,...
  'position'                  ,[baseHpos+Fitbuttonwidth+50,10,Fitbuttonwidth,25], ...
  'tag'                       , 'pushbutton_Load');
uicontrol(...
  'style'                     , 'pushbutton', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , figH, ...
  'string'                  ,   'Save Results',...
  'callback'                , @pushbutton_Save_Callback, ...
  'position'                  ,[baseHpos+Fitbuttonwidth+50, 35,Fitbuttonwidth,25], ...
  'tag'                       , 'pushbutton_Save');
uicontrol(...
  'style'                     , 'pushbutton', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , figH, ...
  'string'                  ,   'Fit',...
  'callback'                , @pushbutton_Fit_Callback,...
  'position'                  ,[baseHpos+200, 60,Fitbuttonwidth,25], ...
  'tag'                       , 'pushbutton_Fit');
uicontrol(...
  'style'                     , 'pushbutton', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'center', ...
  'parent'                    , figH, ...
  'string'                  ,   'Simulate',...
  'callback'                , @pushbutton_simulate_Callback,...
  'position'                  ,[baseHpos, 60,Fitbuttonwidth,25], ...
  'tag'                       , 'pushbutton_simulate');


for i=1:numel(pnames)
    position(1) = baseHpos + 20;
    position(2) = 20*(i-1) + 25 + baseVpos;
    label = pnames{i};
    if isempty(strfind(label, 'option'))
        generate_uifit(position, label, bestP.(pnames{i}), i, figH);
    end
end
    end

% ==============================
% generate a set uicontrol for a parameter.
    function generate_uifit(position, label, value, indx, figH)
pLabel.h = 20;
pVal.h = pLabel.h;
pFit.h = pLabel.h;
pLB.h = pLabel.h;
pUB.h = pLabel.h;

pLabel.w = 50;
pVal.w = 80;
pFit.w = 40;
pLB.w = 60;
pUB.w = 60;

uicontrol(...
  'style'                     , 'text', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'left', ...
  'string'                  , label, ...
  'parent'                    , figH, ...
  'position'                  ,[position(1), position(2),pLabel.w,pLabel.h], ...
  'tag'                       , ['text_p', num2str(indx)]);

uicontrol(...
  'style'                     , 'edit', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'left', ...
  'string'                  , sprintf('%0.3f', value(1)), ...
  'parent'                    , figH, ...
  'position'                  ,[position(1)+pLabel.w+10, position(2),pVal.w,pVal.h], ...
  'tag'                       , ['edit_p', num2str(indx)]);

uicontrol(...
  'style'                     , 'checkbox', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'left', ...
  'string'                  , 'Fit', ...
  'parent'                    , figH, ...
  'position'                  ,[position(1)+pLabel.w+pVal.w + 20, position(2),pFit.w,pFit.h], ...
  'tag'                       , ['checkbox_Fit', num2str(indx)]);


uicontrol(...
  'style'                     , 'edit', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'left', ...
  'parent'                    , figH, ...
  'position'                  ,[position(1)+pLabel.w+pVal.w + pFit.w + 30, position(2),pLB.w,pLB.h], ...
  'tag'                       , ['edit_LB', num2str(indx)]);

uicontrol(...
  'style'                     , 'edit', ...
  'backgroundcolor'           , [1,1,1], ...
  'horizontalalignment'       , 'left', ...
  'parent'                    , figH, ...
  'position'                  ,[position(1)+pLabel.w+pVal.w + pFit.w + pLB.w + 40, position(2),pUB.w,pUB.h], ...
  'tag'                       , ['edit_UB', num2str(indx)]);
    end


function pushbutton_simulate_Callback(varargin)
% hObject    handle to pushbutton_simulate (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% fit parameter reading
p = BLFit_readparameters(handles);

fit = evalin('base', 'fit');
loadfitparam = 0 ;
% fit parameter reading
if sum(p) == 0
    loadfitparam = 1;
end
if sum(isnan(p))
    loadfitparam = 1;
end
if loadfitparam
    p = fit.param;
    BLFit_setparameters(handles, p);
end

y = fit.yd;
cv = fitfunction(p, y, Rmat, []);
Iq = evalin('base', 'Iq');
set(fit.simlineh, 'ydata', Iq);
end


function pushbutton_Fit_Callback(varargin)
% hObject    handle to pushbutton_Fit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

fit = evalin('base', 'fit');
if isfield(fit, 'fitlineh')
    try
        delete(fit.fitlineh)
    catch
        fit.fitlineh = [];
    end
end

roi = fit.roi;
for i=1:numel(fit.data)
    data2fit = fit.data{i};
    fit.fitlineh = line('xdata', data2fit(roi(1):roi(2),1), 'ydata', ...
        data2fit(roi(1):roi(2),2), 'color','r', ...
        'parent', findobj(fit.figh, 'type', 'axes'));
end
assignin('base', 'fit', fit)

if nargin < 3
    handles = fit.handles;
end


% fit parameter reading ==============================
[p, qfit, LB, UB] = BLFit_readparameters(handles);
% ====================================================

y = fit.yd;
roi = [];
if isfield(fit, 'roi')
    if numel(fit.roi) == 2
        roi = fit.roi;
        y = y(roi(1):roi(2));
    end
end

% Initialize parameter to fit ======
NLPstart = p';
% ==================================

options = optimset('fminsearch');
options = optimset(options, 'TolX',0.0000001);
options = optimset(options, 'PlotFcns',@optimplotx);
options = optimset(options, 'OutputFcn',@BLFit_outfun);
options = optimset(options, 'MaxIter',500);

% Vary or not ===========
for i=1:numel(p)
    if ~qfit(i)
        LB(i) = p(i);
        UB(i) = p(i);
    end
end
% ==========================

INLP = fminsearchcon(@(x) fitfunction(x,y, roi),NLPstart,LB,UB, [], [], [], options);
fit.param = INLP;
assignin('base', 'fit', fit);

% Return results ===========
BLFit_setparameters(handles, INLP)
end


function pushbutton_Save_Callback(varargin)
% hObject    handle to pushbutton_Save (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
fn = get(handles.edit_savefilename, 'string');
BLFit_save(handles, fn)
end

function pb_loaddata_Callback(varargin)
% hObject    handle to pb_loaddata (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
BLFit_Loaddata(handles)
BLFit_pickFigure(str2double(get(handles.edit_Figure, 'string')), handles)
end
function pb_LL_Callback(varargin)
% hObject    handle to pb_LL (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
hObject = varargin{1};
BLFit_Indexcallback(hObject, handles)
end
function pb_LU_Callback(varargin)
% hObject    handle to pb_LU (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
hObject = varargin{1};
BLFit_Indexcallback(hObject, handles)
end
function pb_RL_Callback(varargin)
% hObject    handle to pb_RL (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
hObject = varargin{1};
BLFit_Indexcallback(hObject, handles)
end
function pb_RU_Callback(varargin)
% hObject    handle to pb_RU (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
hObject = varargin{1};
BLFit_Indexcallback(hObject, handles)
end
function pushbutton_qselect_Callback(varargin)
% hObject    handle to pushbutton_qselect (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
BLFit_qselect
end
function edit_Lindex_Callback(varargin)
% hObject    handle to edit_Lindex (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit_Lindex as text
%        str2double(get(hObject,'String')) returns contents of edit_Lindex as a double
BLFit_index_callback
end
function edit_Rindex_Callback(varargin)
% hObject    handle to edit_Rindex (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit_Rindex as text
%        str2double(get(hObject,'String')) returns contents of edit_Rindex as a double
BLFit_index_callback
end 
function edit_Lq_Callback(varargin)
% hObject    handle to edit_Lq (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit_Lq as text
%        str2double(get(hObject,'String')) returns contents of edit_Lq as a double
BLFit_qindex_callback
end
function edit_Rq_Callback(varargin)
% hObject    handle to edit_Rq (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit_Rq as text
%        str2double(get(hObject,'String')) returns contents of edit_Rq as a double
BLFit_qindex_callback
end
function edit_Figure_Callback(varargin)
% hObject    handle to edit_Figure (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit_Figure as text
%        str2double(get(hObject,'String')) returns contents of edit_Figure as a double
Figh = str2double(get(hObject, 'string'));
BLFit_pickFigure(Figh, handles)
end

function pushbutton_Load_Callback(varargin)
% hObject    handle to pushbutton_Load (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
BLFit_LoadSaved(handles)
end

function cv = fitfunction(p, y, roi)
fit = evalin('base', 'fit');
fit.param = p;

if isempty(roi)
    q = fit.xd;
else
    q = fit.xd(roi(1):roi(end));
end
q = q(:);


% Calculation function
[predY, fit] = multivoight(parameters, q);

cv = chi_squared(y, predY, 5);
fit.fit = [q(:), predY(:), y(:)];
assignin('base', 'fit', fit)
end














end