function Z = LatticeCal(qx, qy, qz, a1, a2, a3, gfactor)
% function LatticeCal(qx, qy, qz, a1, a2, gfactor, N1, N2)
% this is the same function as BCCLattice.m ---- calculation for
% paracrystal
% but, it is generallized to every lattice, where a1, a2, a3 should be
% fundamental vectors of any lattice.
% this function calculate the paracrystal lattice factor with distortion of
% the second kind  ....... ref : Hashimoto et al. Macromolecules, Vol 27, 11,
% 1994, 3063
%
% finite size of domain is assumed in direction of a1 ==> N1
%                                                  a2 ==> N2 
%                                                  a3 ==> N3
% a1 and a2 should be a vector of cartesian coordinate...
% gfactor = [g11, g12, g21, g22];
% q could be matrix.

N1 = 1000;N2 = 1000; N3 = 1000;

qx= real(qx);qy = real(qy);qz=real(qz);     % Lattice factorÿ real ..

g = gfactor(1);

qa1 = a1(1)*qx + a1(2)*qy + a1(3)*qz;
qa2 = a2(1)*qx + a2(2)*qy + a2(3)*qz;
qa3 = a3(1)*qx + a3(2)*qy + a3(3)*qz;

q = real(sqrt(qx.^2 + qy.^2 + qz.^2));
absF1 = abs(exp(-1/2*(g^2 * qa1.^2 + g^2 * qa2.^2 + g^2 * qa3.^2)));
F1 = absF1.*exp(-j*qa1);
Z1 = real((1 + F1)./(1 - F1+eps));
Ic1 = -2*real(F1.*(1-F1.^N1)./((1-F1).^2 + eps));

absF2 = abs(exp(-1/2*(g^2 * qa1.^2 + g^2 * qa2.^2 + g^2 * qa3.^2)));
F2 = absF2.*exp(-j*qa2);
Z2 = real((1 + F2)./(1 - F2+eps));
Ic2 = -2*real(F2.*(1-F2.^N2)./((1-F2).^2 + eps));

absF3 = abs(exp(-1/2*(g^2 * qa1.^2 + g^2 * qa2.^2 + g^2 * qa3.^2)));
F3 = absF3.*exp(-j*qa3);
Z3 = real((1 + F3)./(1 - F3+eps));
Ic3 = -2*real(F3.*(1-F3.^N3)./((1-F3).^2 + eps));

Z = (Z1 + Ic1/N1).*(Z2 + Ic2/N2).*(Z3 + Ic3/N3);
%Z = (Z1.*Z2.*Z3);