#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

# include "mex.h"
# include "math.h"
# include <stdio.h>
# define MAX_nvertices_per_face 12

typedef struct{
	double i, j, k;
} Vector;

typedef struct{
	double x, y, z;
} Q;

struct notReal {
    double r, i;
};

Vector VectorAdd(Vector a, Vector b)
{
	Vector c = { a.i + b.i, a.j + b.j, a.k + b.k };
	return c;
}

Vector VectorSub(Vector a, Vector b)
{
	Vector c = { a.i - b.i, a.j - b.j, a.k - b.k };
	return c;
}

Vector VectorScalarDiv(Vector a, double b)
{
    Vector c = {a.i/b, a.j/b, a.k/b};
    return c;
}

double VectorNorm(Vector a)
{
	return sqrt(a.i*a.i + a.j*a.j + a.k*a.k);
}

double dotProduct(Vector a, Vector b)
{
	return a.i*b.i + a.j*b.j + a.k*b.k;
}

Vector crossProduct(Vector a, Vector b)
{
	Vector c = { a.j*b.k - a.k*b.j, a.k*b.i - a.i*b.k, a.i*b.j - a.j*b.i };

	return c;
}

double scalarTripleProduct(Vector a, Vector b, Vector c)
{
	return dotProduct(a, crossProduct(b, c));
}


struct notReal line_integral(Vector n1, Vector q, double qtan, Vector vt1, Vector vt2)
{
	double length_edge = 0.0;
	struct notReal C_integral = {0.0, 0.0};
	struct notReal phasefactor2 = {0.0, 0.0};
	struct notReal phasefactor1 = {0.0, 0.0};
	double triplescalarprod = 0.0;
    double vq = 0.0;
	double qT = 0.0;
	Vector T;

	length_edge = VectorNorm(VectorSub(vt2, vt1));
	T = VectorScalarDiv(VectorSub(vt2, vt1), length_edge);
    vq = dotProduct(vt2, q);
	phasefactor2.r = cos(vq);
    phasefactor2.i = sin(vq);
	triplescalarprod = scalarTripleProduct(T, q, n1);
	C_integral.r = -1*length_edge*triplescalarprod*phasefactor2.i;
	C_integral.i = length_edge*triplescalarprod*phasefactor2.r;
	qT = dotProduct(T, q);
	if (qT == 0.0)
	{
        vq = dotProduct(vt1, q);
		phasefactor1.r = cos(vq);
        phasefactor2.i = sin(vq);
		C_integral.r = triplescalarprod / qT*(phasefactor2.r - phasefactor1.r);
		C_integral.i = triplescalarprod / qT*(phasefactor2.i - phasefactor1.i);
	}
	else
    {
		C_integral.r = C_integral.r / (qtan*qtan);
		C_integral.i = C_integral.i / (qtan*qtan);
    }
	return C_integral;
}

double triangle_area(Vector a, Vector b, Vector c)
{
	return VectorNorm(crossProduct(VectorSub(b, a), VectorSub(c, a))) / 2.0;
}

double tri_area(double *vertices)
{
	Vector a, b, c;

	a.i = *(vertices);
	a.j = *(vertices + 1);
	a.k = *(vertices + 2);
	b.i = *(vertices + 3);
	b.j = *(vertices + 4);
	b.k = *(vertices + 5);
	c.i = *(vertices + 6);
	c.j = *(vertices + 7);
	c.k = *(vertices + 8);
	return triangle_area(a, b, c);
}

double face_area(double *vertices, int n_vertices)
{
	double tri[9] = {0.0};
	double area = 0.0;
    int k;

	if (n_vertices < 3)
		return -1.0;

	if (n_vertices == 3)
	{ 
		return tri_area(vertices);
	}
	else {
		for (k = 0; k < n_vertices - 2; k++)
		{
			tri[0] = *(vertices);
			tri[1] = *(vertices + 1);
			tri[2] = *(vertices + 2);
			tri[3] = *(vertices + (k + 1) * 3);
			tri[4] = *(vertices + (k + 1) * 3 + 1);
			tri[5] = *(vertices + (k + 1) * 3 + 2);
			tri[6] = *(vertices + (k + 1) * 3 + 3);
			tri[7] = *(vertices + (k + 1) * 3 + 4);
			tri[8] = *(vertices + (k + 1) * 3 + 5);
			area = area + tri_area(tri);
		}
		return area;
	}
}

struct notReal surfae_integral(double *vertices, int nvertices, Vector n, Vector q)
{
	/* Calculate integral for a face, whose number of vertices is nvertices.*/
	/* vertices is the vertices of the face.*/
	/* n is the face normal vector.*/

	double area_k;
	double nq;
	struct notReal phasefactor, sk, lint;
	Vector vt1, vt2;
	double qtan, vq;
	struct notReal skline = {0.0, 0.0};
    int v_indx;
    
    sk.r = 0.0;
    sk.i = 0.0;
    
	nq = dotProduct(n, q);

	if (nq == 0.0)
	{
		area_k = face_area(vertices, nvertices);
		vt1.i = vertices[0];
		vt1.j = vertices[1];
		vt1.k = vertices[2];
        vq = dotProduct(vt1, q);
		phasefactor.r = cos(vq);
        phasefactor.i = sin(vq);
        sk.r = nq*area_k*phasefactor.r;
        sk.i = nq*area_k*phasefactor.i;
		return sk;
	}
	else
	{
		qtan = VectorNorm(crossProduct(q, n));
		if (qtan == 0.0)
		{
			return sk;
		}
		else 
		{
			skline.r = 0.0;
            skline.i = 0.0;
			for (v_indx = 0; v_indx < nvertices; v_indx++)
			{
				if (v_indx < nvertices - 1)
				{
					vt1.i = vertices[v_indx * 3];
					vt1.j = vertices[v_indx * 3 + 1];
					vt1.k = vertices[v_indx * 3 + 2];
					vt2.i = vertices[(v_indx + 1) * 3];
					vt2.j = vertices[(v_indx + 1) * 3 + 1];
					vt2.k = vertices[(v_indx + 1) * 3 + 2];
				}
				else {
					vt1.i = vertices[v_indx * 3];
					vt1.j = vertices[v_indx * 3 + 1];
					vt1.k = vertices[v_indx * 3 + 2];
					vt2.i = vertices[0];
					vt2.j = vertices[1];
					vt2.k = vertices[2];
				}
				lint = line_integral(n, q, qtan, vt1, vt2);
                skline.r = skline.r + lint.r;
                skline.i = skline.i + lint.i;
			}
            sk.r = nq*skline.r;
            sk.i = nq*skline.i;
			return sk;
		}
	}
}


void mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	/*_diaz(nfaces, nvert_face, normalvectors, allfaces, allvertices, Q*/
	/* nfaces : total number of faces, int*/
	/* nvert_Faces : array of the number of vertices for each faces, int matrix. The number of element is nfaces.*/
	/* allfaces : array of the indices of vertices. int*/
	/* allvertices : array of all vertices position. double*/
    /* circularavg(image, offset, [beamX, beamY], [maxlimit, minlimit], [mu_start, mu_end], mask, Qoption, [Qmax, Qnum], waveln, psize, SDD);
    // default: maxlimit = 2^16, minlimit = -2^16
    //          azimthalangle = []
    //          mask =[];
    //          Qoption = 0 // pixel based calculation
    //          Qscale = [Qmax, number of Q] -- largest Q, and number of Q....
     */
    
	/*mxArray *img_, *bxy_, *scale_, *mu_, *mask_, Qscale_;*/
    double *allvertices, *normalvectors, *Q;
	double vertices[MAX_nvertices_per_face*3] = {0.0};
	struct notReal sk, skret;
	int nfaces, nvertices, *nvert_faces, *allfaces;
    int faceindex0=0;
	Vector n, q;
	double *Apr, *Api;
	long nQ;
    int vindex;
    int m, i;
    long k;

	if (nrhs != 5) {
		mexPrintf("%i\n", nrhs);
		mexErrMsgTxt("5 inputs are required.");
	}
	if (nlhs != 1)
		mexErrMsgTxt("1 output is required.");

	nfaces = (int) mxGetScalar(prhs[0]);
	*nvert_faces = (int) mxGetPr(prhs[1]);
	normalvectors = mxGetPr(prhs[2]);
	*allfaces = (int) mxGetPr(prhs[3]);
	allvertices = mxGetPr(prhs[4]);
	Q = mxGetPr(prhs[5]);
	nQ = (long) mxGetNumberOfElements(prhs[5]);

	plhs[0] = mxCreateDoubleMatrix(nQ, 1, mxCOMPLEX);
	Apr = mxGetPr(plhs[0]);
	Api = mxGetPi(plhs[0]);

	for (k = 0; k < nQ; k++)
	{
		q.i = Q[k * 3];
		q.j = Q[k * 3 + 1];
		q.k = Q[k * 3 + 2];
		faceindex0 = 0;
		sk.r = 0.0;
        sk.i = 0.0;
		for (i = 0; i < nfaces; i++)
		{
			n.i = normalvectors[i * 3];
			n.j = normalvectors[i * 3 + 1];
			n.k = normalvectors[i * 3 + 2];
			nvertices = nvert_faces[i];
			for (m = 0; m < nvertices; m++)
			{
				vindex = allfaces[faceindex0 + m];
				vertices[m * 3] = allvertices[vindex * 3];
				vertices[m * 3 + 1] = allvertices[vindex * 3 + 1];
				vertices[m * 3 + 2] = allvertices[vindex * 3 + 2];
			}
			skret = surfae_integral(vertices, nvertices, n, q);
            sk.r = sk.r + skret.r;
            sk.i = sk.i + skret.i;
			faceindex0 = faceindex0 + nvertices;
		}
		Apr[k] = sk.r;
		Api[k] = sk.i;
	}
}