function [Formfactor, name, pnames, pin] = cylinder_vert_type_matrix(qp, qz, R, H, nR, vf)
% this is Amplitude for cylinder type particle : cylindertype(qp, qz, p)
% cylinder is assumed to normal to surface.
% Hard sphere potential is applied.....
% p = [R, L, nR]  for an array of R used.
% p = [R, L, I0]  for an single R.
% 2004. Feb, 4
[xq, yq] = size(qp);
lenR = length(R);
%if lenR < 40;lenR = 40;end
%R = R(1:fix(lenR/40):end);
%I0 = I0(1:fix(lenR/40):end);
if xq < yq 
    qp = qp';     % q should be column vector
    qz = qz';
end
qp = real(qp);qz = real(qz);

if (length(R) ~=1) & (length(H) == 1)
    q = qp;
    R0 = R;
    [R, qp] = meshgrid(R, q);
    [I0, temp] = meshgrid(nR, q);
    H = H*ones(size(R));
    qz = qz*ones(size(R));
elseif (length(R) ==1) & (length(H) ~= 1)
    q = qz;
    H0 = H;
    [H, qz] = meshgrid(H, q);
    I0 = nR;
    %[I0, temp] = meshgrid(I0, q);
    R = R*ones(size(H));
    qp = qp*ones(size(H));
end
F = 2*pi*R.^2.*H.*besseljc(qp.*R).*sinc(qz.*H/2);
Fnorm = besseljc(qp.*R).*sinc(0.0001.*H/2); % particle volume dependent Intensity !!!!
if vf ~= 0
    Struc = SFactor(q, R0, vf);
    %F = sum(I0.*abs(F).^2, 2);
    F = sum(I0.*abs(F).^2.*Struc, 2);
else
    %size(F)
    %size(I0)
    F = sum(I0.*abs(F).^2, 2);
end

Fnorm = sum(abs(Fnorm).^2, 2);
F = F/max(abs(Fnorm));
Formfactor = F;

function SF = SFactor(q, Rhs, vf)
% vf is volume fraction of hard sphere
alpha = (1 + 2*vf)^2/(1-vf)^4;
beta = -6*vf*(1 + vf/2)^2/(1-vf)^4;
gamma = vf*alpha/2;

if length(Rhs) > 1
    [R, q] = meshgrid(Rhs, q);
    A = R.*q*2;
else
    A = Rhs.*q*2;  
end
A = A+eps;
GA = alpha*(sin(A)-A.*cos(A))./A.^2 + ...
    beta*(2.*A.*sin(A) + (2 - A.^2).*cos(A)- 2)./A.^3 + ...
    gamma*(-1*A.^4.*cos(A) + 4*((3*A.^2-6).*cos(A) + (A.^3 - 6 * A).*sin(A) + 6))./A.^5;
SF = 1./(1+24*vf*GA./A);