function [fq, avFq2] = hollowCylinderForm(qr, R1, R2, R3, h1, h2)
% Formfactor of hollow cylinder
% [fq, avFq2] = hollowCylinderForm(qr, R1, R2, R3, h1, h2)
% currently, only fq are calculated, avFq2 = abs(fq).^2;
% D.R. Lee et al. APL, 82, 6, 982

if R2 > R3
    dr = (R2-R3)/100;
    r = R3:dr:R2;   %R2 > R3
    alpha = (h1-h2)/(R2-R3);

elseif R2 < R3;
    temp = R3;
    R3 = R2;
    R2 = temp;
    alpha = (h1-h2)/(R2-R3);

else
    r = R2;
    dr = 0;
    alpha = 0;
end

[nx, ny] = size(qr);

if nx<ny
    qr = qr';
end
fq = 2*pi*h1*R1./qr.*bessel(1, qr*R1) + 2*pi*alpha./qr.^2.*(R2*bessel(0, qr*R2) - R3*bessel(0, qr*R3) - sum(bessel(0, qr*r)*dr, 2));
avFq2 = abs(fq).^2;