function [Intensity, F1, F2] = intenBCC(q, R, sig, sigs, a, gfactor)
% [Intensity, F1, F2] = intenBCC(q, R, sig, sigs, a, gfactor)
% Intensity : scattering intensity from the unit volume of sample
% R : average radius of particle
% sig : standard deviation of radiuses of particles
% sigs : surface smoothness of particle
% v: overall volume of particle(actually, this is N*v, here, overall volume
%    is unit volume then v = 1)
% a : interparticle distance
% gfactor : ordering paramter gfactor = da / |a|

[gDist, xx] = gaussdist99(R, sig);
[x1, x2]= size(q);
    
if x1 < x2
    q = q';
end

F = sphereamp(q, xx, sigs);

if length(gDist) == 1
    step = 1
else
    step = (xx(end) - xx(end-1));
end

F1 = sum((F*gDist).^2, 2)*step;  % square of average of F
F2 = sum(F.^2*gDist, 2)*step; % average of F square

th = 0:pi/120:pi;
ph = 0:pi/60:(2*pi);

CrystalType = 'BCC';
dela = gfactor*a;
Z = integLattice2(q, a, dela, th, ph, CrystalType);Z = Z';

v = 1;
Intensity = (F2 - F1) + 1/v*F1.*Z; 