function [y, name, pnames, pin]=mld(q,p, flag)
% mld     : multilayerdisk
% [y, {name, pnames, pin}]=mld(q,p, {flag}) 
% MFIT multilayerdisk fitting function
% p = [ core_radius, core_thick, shell_thick, density, I0, back ]

% Author:  MZ <mzinkin@sghms.ac.uk>
% Description: Gaussian

if nargin==2;
    if length(p) > 12   % two gaussian model
        totallen = max([p(2), p(5)]); % zh, position of a head group
        totallen = totallen*2;
        sh_thick = 1;
        shell = 0:sh_thick:totallen;
        yy = ebilayer3(shell, p);
    else            % single gaussian model
        totallen = p(2); % zh, position of a head group
        totallen = totallen*2;
        sh_thick = 1;
        shell = 0:sh_thick:totallen;
        yy = ebilayer2(shell, p);
    end
    
    xx = shell;eprofile=yy;
    shell(1:2) = [];
    core_thick = sh_thick*2;
    shell_thick = sh_thick*ones(size(shell));
    density = yy;
    core_radius = p(end-3);
    sigofR = p(end-2);
    I0 = p(end-1);
    back = p(end);
    
    y = multilayerdisk(q, core_radius, core_thick, shell_thick, density, sigofR, I0, back);
else
	y=[];
	name='multilayerdisk';
	pnames=str2mat('Ah', 'zh', 'sigh','Am', 'sigm', 'ed/methylene', 'de/water', 'radius','sigofR','I0', 'back');
	if flag==1, pin=[1  17 3 -1  3 -0.1, 0, 150, 0, 1, 0]; else pin = p; end
	end
end
