function [fq, avFq2, av2Fq] = orientCylinder(q, alpha, R, RLratio, sigS, sigR)
% function [fq, avFq2, av2Fq] = orientCylinder(q, alpha, R, L, sigS)
% alpha : angle between q and n vector(cylinder axis direction vector)
% R : Radius of cylinder
% RLration : L / R value
% sigS : a parameter characterizing the interface thickness of the
%   cylinders along their radii
% sigR : deviation of radius R
[oSizex, oSizey] = size(q);
q = reshape(q, 1, prod(size(q)));      % q => [x x x x x x x x] form
alpha = reshape(alpha, 1, prod(size(alpha)));alpha = deg2rad(alpha);

if prod(size(alpha)) ~= prod(size(q))           % if number of element of alpha is 1 and not same with that of q....
    if prod(size(alpha)) == 1
        alpha = alpha*ones(1, prod(size(q)));       % make same number of element of alpha....
    end
end
q = q+eps;
alpha = alpha + eps;
if sigR == 0
    L = RLratio*R;
    fq = 2*pi*R.^2.*L.*besselj(1, R.*q.*sin(alpha))./(R.*q.*sin(alpha)).*sinc(L.*q.*cos(alpha)/2);
    avFq2 = abs(fq).^2;
    av2Fq = avFq2;
else
    if R > 5
        [P_R, Rdist] = gaussd(R, sigR);P_Rmat = P_R*ones(1, length(q));
    else
        interval = R/5;
        [P_R, Rdist] = gaussd(R, sigR);P_Rmat = P_R*ones(1, length(q));
    end
    q(find(q == 0)) = 0.0000001;
    L = RLratio*Rdist;
    fq = 2*pi*Rdist.^2.*L.*besselj(1, Rdist*(q.*sin(alpha)))./(Rdist*(q.*sin(alpha))).*(sinc(L*(q.*cos(alpha))/2));
    avFq2 = sum(P_Rmat .* abs(fq).^2, 1) ./ sum(P_R);
    av2Fq = (sum(P_Rmat .* abs(fq), 1) ./sum(P_R)).^2;
    fq = sum(P_Rmat .* abs(fq), 1) ./ sum(P_R);
end

fq = reshape(fq, oSizex, oSizey);
avFq2 = reshape(avFq2, oSizex, oSizey);
av2Fq = reshape(av2Fq, oSizex, oSizey);

function [gDist, xx] = gaussd(center, width, interval)
% gaussian distribution generator, 99% 
% gDist, xx will be column vector
if nargin < 3
    xx = 0.01:1:(center+abs(width)*3);
    if width == 0
        gDist = 1;
        xx = center;
    else
        if abs(width) < center
            gDist = normpdf(xx, center, abs(width))';
            cutoff = find(gDist < 0.03*max(gDist));
            gDist(cutoff) = [];  xx(cutoff) = [];
            xx = xx';
            gDist = gDist/sum(gDist);
        else
            gDist = -1;
            xx = center;
        end
    end
else
    xx = 0.01:interval:dmax;
    if width == 0
        gDist = 1;
        xx = center;
    else
            gDist = normpdf(xx, center, abs(width))';
            xx = xx';
            gDist = gDist/sum(gDist);
    end
end   