function [Z, b1, b2, b3] = paraBCC3(qx, qy, qz, b1, b2, b3, RotAxis, Rotangle, Dsp, delA)
% Z = paraBCC3(qx, qy, qz, b1, b2, b3, RotAxis, Rotangle, delA)
% paracrystalline BCC lattice factor
% b1, b2, b3 should be determined for lattice orientation.
% Lattice determined by b1, b2, b3 can be rotated as 'Rotangle' about
% the axis of 'Rotaxis'.
% delA is the distortion factor.
%
% Assumption 
%  1. |a1| = |a2| = |a3| = a, here denoted as A.
%  2. isotropic distortion.

b1 = b1*Dsp ;
b2 = b2*Dsp ;
b3 = b3*Dsp;
a1 = 1/2*(-b1+b2+b3);
a2 = 1/2*(b1-b2+b3);
a3 = 1/2*(b1+b2-b3);

a1 = eulerrot2(a1, RotAxis, Rotangle);
a2 = eulerrot2(a2, RotAxis, Rotangle);
a3 = eulerrot2(a3, RotAxis, Rotangle);
b1 = eulerrot2(b1, RotAxis, Rotangle);
b2 = eulerrot2(b2, RotAxis, Rotangle);
b3 = eulerrot2(b3, RotAxis, Rotangle);

qx = real(qx);qy = real(qy);qz = real(qz);

Z = BCClattice(qx, qy, qz, a1, a2, a3, delA/Dsp);
