function [y, name, pnames, pin] = pedersen_Cmulti(x, p, flag)
% [y, name, pnames, pin] = pedersen_Cmulti(x, p, flag)
% Pedersen formalism to calculate scattering intensity.
% Local monodisperse approximation assumed.
% Formfactor assumed as cylinder parallel scattering.
% size distribution function : Gaussian(0), Gamma(1)
% Structure factor : Hard sphere
global nbasis flag_aw

if nargin == 2;
    I0 = p(1);
    volfrac = abs(p(2)); % always positive
    H = abs(p(3)); % always positive
    Rmin = abs(p(4));  % always positive
    Rmax = p(5);
    nbasis = p(6);
    for tt=1:nbasis
        pa(tt) = p(tt+6);
    end
    
    [gDist, xx] = multi_gauss(Rmin, Rmax, pa);
    
    [x1, x2]= size(x);
    
    if x1 < x2
        x = x';
    end
    %normFactor = sum(sum((gDist*xx.^6)));size(normFactor);%SS = SFactor(x, xx, volfrac);
    %y = sum(spheretype(x, xx).*SS*(gDist*xx.^6), 2)*I0/normFactor + bkg ;
    %normFactor = gDist'*(xx.^6);
    normFactor = sum(gDist);

    y = zeros(size(x));
%    figure;hold on
    for i = 1:length(xx)
        %Form = spheretype(x, xx(i))*xx(i)^6;  % let's discribe using volume distribution... because I don't want exaggerate small size particle...
        Form = cylinder_vert_type(x, 0.1, [xx(i), H, 1]);Form = abs(Form).^2;Form = Form/max(Form);
        Struc = SFactor(x, xx(i), volfrac);
        y = y + Form.*Struc.*gDist(i);
        %y = y + Form.*gDist(i);
%        plot(x, Form, 'b');plot(x, Struc, 'r');
    end
    y = y*I0/normFactor+0;

else
	y=[];
	name='Pedersen Cylinder';
    if size(flag_aw,2) < 14
      flag_aw = [flag_aw zeros(1,14)];
   end;

    if flag_aw(1:14) ~= 'pedersen_multi'
      nbasis=input('Number of basis functions:');
      flag_aw = str2mat('pedersen_multi');
   end;
   
    pnames = str2mat('I0','Volumefraction','H', 'Rmin', 'Rmax', 'numberofbais');
    for tt=1:nbasis
        pnames = str2mat(pnames, sprintf('p_%d',tt));
    end
	if flag==1, pin=[1 0 20 1 300 nbasis 0 ones(1, nbasis-2) 0]; else pin = p; end
end

function SF = SFactor(q, Rhs, vf)
% vf is volume fraction of hard sphere
alpha = (1 + 2*vf)^2/(1-vf)^4;
beta = -6*vf*(1 + vf/2)^2/(1-vf)^4;
gamma = vf*alpha/2;

if length(Rhs) > 1
    [R, q] = meshgrid(Rhs, q);
    A = R.*q*2;
else
    A = Rhs.*q*2;  
end

GA = alpha*(sin(A)-A.*cos(A))./A.^2 + ...
    beta*(2.*A.*sin(A) + (2 - A.^2).*cos(A)- 2)./A.^3 + ...
    gamma*(-1*A.^4.*cos(A) + 4*((3*A.^2-6).*cos(A) + (A.^3 - 6 * A).*sin(A) + 6))./A.^5;
SF = 1./(1+24*vf*GA./A);
%plot(SF(:,1))