function [F, pv] = saxsPoints2Amp(qx,qy,qz, pv, V)
% calculate F(q) from pv. 
% pv format: m x 3
% pv should be lattice points of uniform rectangular grid to calculate the
% volume of the object.

% if volume is missing,
Np = numel(pv(:,1));
if nargin < 5
    x = pv(:,1);
    x = unique(x);
    dx = x(2) - x(1);
    y = pv(:,2);
    y = unique(y);
    dy = y(2) - y(1);
    z = pv(:,3);
    z = unique(z);
    dz = z(2) - z(1);
    V = dx*dy*dz*Np;
end

j = sqrt(-1);
Q = [qx(:),qy(:),qz(:)];

for i=1:numel(qx)
    F(i) = sum(exp(-j*Q(i,:)*pv'),2);
end
F = reshape(F, size(qx))/Np*V;
