function [F, pv] = saxsPolyhedronAmp(qx,qy,qz, obj, forOrgn)
% [F, pv] = saxsPolyhedronAmp(qx,qy,qz, obj, forOrgn)
% if obj is m x 3 points comprising an object, this code will use
% saxsPoints2Amp.m to compute scattering amplitude F. Otherwise, obj should
% be a struct of a polyhedron. Then it will produce a matrix of m x 3
% points comprising the object and compute F.
% see also saxsPolyhedron.m and saxsPoints2Amp.m

if ~isstruct(obj)
    F = saxsPoints2Amp(qx,qy,qz, obj);
    pv = obj;
    return
end

if nargin < 5
    forOrgn = 0;
end

CM = mean(obj.vertices);
nv = obj.vertices - repmat(CM, numel(obj.vertices(:,1)), 1);
minV = min(nv, [], 1);
maxV = max(nv, [], 1);
if isfield(obj, 'stellate')
    res = 40;
else
    res = 30;
end

x = linspace(minV(1), maxV(1), res) + CM(1);
y = linspace(minV(2), maxV(2), res) + CM(2);
z = linspace(minV(3), maxV(3), res) + CM(3);

dx = x(2) - x(1);
dy = y(2) - y(1);
dz = z(2) - z(1);
dv = dx*dy*dz;
%x = linspace(-mdist, mdist, 50);
[X, Y, Z] = ndgrid(x, y, z);
pv = [X(:), Y(:), Z(:)];
%pv = (rand(50^3, 3)-0.5)/0.5*mdist;

%inP = inpolyhedron(obj, pv);
if forOrgn == 1
    if isfield(obj, 'original')
        obj = obj.original;
    end
end
inP = inPolyhedron2(obj, pv);
k = inP==0; pv(k, :) = [];
Np = numel(pv(:,1));
V = dv*Np;
F = saxsPoints2Amp(qx,qy,qz, pv, V);
