function [F, Ftet, Foct] = saxsstellateoctahedron2(qx, qy, qz, RH)
% calculate F of saxsstellateoctahedron by tranforming particles instead of
% transforming coordinates.
% requires model for tetrahedron.
%   tet = polyhedra('tetrahedron'); tet.vertices = tet.vertices*R;
%

if numel(RH) == 1
    R = RH;
    H = sqrt(2/3)*L;    % Lengthes of all edges of this tetrahedron is the same.
else
    R = RH(1);H = RH(2);
end

Foct = saxsoctahedron(qx,qy,qz,R);
%Foct = saxsPolyhedronAmp(q1,q2,q3,tet);



tet = evalin('base', 'tet');
sinth = sqrt(2/3);
%costh = 1/sqrt(3);
ang = asin(sinth);
angRot = [ang, ang+pi];
%angRot = [ang];
angIn = pi/2*[0,1,2,3];

if numel(size(qx))>2
    Ftet = zeros(size(qx));
else
    Ftet = zeros(size(qx,1), size(qx,2), 8);
end


for l=1:numel(angRot)
    ang = angRot(l);
    for m=1:numel(angIn)
        angxy = angIn(m);
        
        MRP = [1,0,0;0,cos(ang), -sin(ang);0, sin(ang), cos(ang)]; % for Rotation for Particle.
        MRPxy = [cos(angxy), -sin(angxy) 0;sin(angxy), cos(angxy) 0; 0,0,1];
        temp = tet;
        temp.vertices = (MRPxy*MRP*tet.vertices')';
        Zt = R/sqrt(3)*sin(ang);
        Yt = -(1-cos(ang)/sqrt(3))*R;
        fprintf('ang, angxy, Zt and Yt = %0.3f, %0.3f, %0.3f and %0.3f\n',ang, angxy, Zt,Yt)
        switch angxy 
            case 0;
                Zm = repmat([0, 0, Zt], size(temp.vertices, 1), 1);
                Ym = repmat([0, -abs(Yt), 0], size(temp.vertices, 1), 1);
            case pi/2
                Zm = repmat([0, 0, Zt], size(temp.vertices, 1), 1);
                Ym = repmat([abs(Yt), 0, 0], size(temp.vertices, 1), 1);
            case pi
                Zm = repmat([0, 0, Zt], size(temp.vertices, 1), 1);
                Ym = repmat([0, abs(Yt), 0], size(temp.vertices, 1), 1);
            case pi/2*3
                Zm = repmat([0, 0, Zt], size(temp.vertices, 1), 1);
                Ym = repmat([-abs(Yt), 0, 0], size(temp.vertices, 1), 1);
        end
        switch l
            case 1
                Zm = Zm;
                Ym = Ym;
            case 2
                Zm = Zm;
                Ym = Ym;
        end
        
        temp.vertices = temp.vertices + Zm + Ym;
        
        tt = 4*(l-1) + m;
        F0 = saxsPolyhedronAmp(qx,qy,qz,temp);
        
        if numel(size(qx))>2
            Ftet = Ftet + F0;
        else
            Ftet(:,:,tt) = F0;
        end
        %drawpolyhedron(temp.vertices, temp.faces, 'r')
    end
end

Vtet = 1.1785e+05;
Ftet = Ftet*Vtet;

if numel(size(qx))>2
    F = Foct + Ftet;
else
    F = Foct + sum(Ftet, 3);
end