function [F, V, S] = saxstetrahedral(qx, qy, qz, l)
% function y = saxstetrahedron(q, l)
% This calculates Iq of only the regular tetrahedral.
% l is the edge length of a triangle;
% reference
%   http://journals.iucr.org/j/issues/2011/03/00/ce5109/ce5109.pdf
% This gives slightly different result at high q than any other
a = 2*sqrt(2);
V = 8/3;

qx = l/a*qx+eps;
qy = l/a*qy+eps;
qz = l/a*qz+eps;
q = sqrt(qx.^2+qy.^2+qz.^2);
T1p = 4*qx.*qy.*qz.*(cos(sqrt(2)*qx)+cos(sqrt(2)*qy));
T1m = 4*qx.*qy.*qz.*(cos(sqrt(2)*qx)-cos(sqrt(2)*qy));
T2p = qy.*(qx.^2-qy.^2+2*qz.^2).*sin(sqrt(2)*qx) + ...
    qx.*(-qx.^2+qy.^2+2*qz.^2).*sin(sqrt(2)*qy);
T2m = qy.*(qx.^2-qy.^2+2*qz.^2).*sin(sqrt(2)*qx) - ...
    qx.*(-qx.^2+qy.^2+2*qz.^2).*sin(sqrt(2)*qy);
%T3 = qx.*qy.*(qx.^4+(qy.^2-2*qz.^2).^2-2*qx.^2.*(qy.^2+2*qz.^2))+(q*100).^3*eps;
T3 = qx.*qy.*(qx.^4+(qy.^2-2*qz.^2).^2-2*qx.^2.*(qy.^2+2*qz.^2));
T3(abs(T3)<eps) = 1;
Tr = T1p.*sin(qz)-sqrt(2)*T2p.*cos(qz);
Ti = T1m.*cos(qz)+sqrt(2)*T2m.*sin(qz);

F = (4./T3).*sqrt(Tr.^2 + Ti.^2)*(l/a)^3;