function [a, obj] = situs2polyhedron(varargin)
% calculate facenormal vector and if it face inward, flip it to face out.
% required to use diazc.m
%
% Example usage;
% a = polyhedron_facecheck_situs('p3o58.situs')
% y2 = saxs_average(q, 'diazc', a);
% Generate the situs file from pdb file using 'pol2vol'
% or Use pdb2sax
% or y2 = debyepr(q, obj.x, obj.y, obj.z);
%
% 6/20/2015
% Byeongdu Lee
situs_filename = varargin{1};
% isolevel = 2;
% if numel(varargin)>1
%     isolevel = varargin{2};
% end
% 
mf = findcellstr(varargin, 'isolevel');
if ~isempty(mf)
    isolevel = varargin{mf+1};
else
    isolevel = 2;
end

mf = findcellstr(varargin, 'maxfacenum');
if ~isempty(mf)
    maxfacenum = varargin{mf+1};
else
    maxfacenum = 1200;
end
inoutcheckNpoint = 5;

% Reading situs file...
fprintf('Loading the situs file ''%s''...\n', situs_filename);
[t, x, y, z] = readsitus(situs_filename);
max_density = max(t(:));
fprintf('Generating isosurface with level/max %0.2f/%0.2f...\n', isolevel, max_density);
fv = isosurface(x, y, z, t, isolevel);
facenum = size(fv.faces, 1);

fprintf('Reduce number of patches using ''reducepatch.m''.\n');
while (facenum > maxfacenum)
    fv = reducepatch(fv, maxfacenum);
    facenum = size(fv.faces, 1);
end
k = t>=isolevel;
x = x(k);
y = y(k);
z = z(k);
a = fv;
obj.x = x;
obj.y = y;
obj.z = z;

fprintf('Start clearing poorly defined faces by ''reducepath.m''.\n');
    faces = a.faces;
    vertices = a.vertices;
% clearing bad faces.    
    [~, ind] = reducehkl(sort(faces, 2));
    faces = faces(ind, :);
    fsize = size(faces);
% Find a vertices included only one face and remove the face and vertex.
    badvertices = [];
    badface = [];
    for i=1:numel(vertices(:,1));
        faceindices_containg_vertex_i = find(faces == i);
        if numel(faceindices_containg_vertex_i) == 1
            fprintf('Vertex %i comprises only one face and thus will be removed.\n', i);
            badvertices = [badvertices, i];
            [badfaceindex, ~] = ind2sub(fsize, faceindices_containg_vertex_i);
            badface = [badface, badfaceindex];
        end
    end
    faces(badface, :) = [];
    vertices(badvertices, :) = [];
    badvertices = sort(badvertices, 'descend');
    for i=1:numel(badvertices)
        ind = faces >= badvertices(i);
        faces(ind) = faces(ind) - 1;
    end
fprintf('Clearing done.\n');

% Now processing....    
    
    Nfaces = numel(faces(:, 1));
    faceNorm = [];
    cm = [];
    ngb_faces = {};
    ngb_manyfaces = [];
    mf = 1;
    
fprintf('Now, computing face properties...\n');
    for i=1:Nfaces
        f1 = faces(i, :);
        cm_k = mean(vertices(f1, :));
        fv = vertices(f1, :);
        n1 = planeNormal([fv(1, :), ...
            fv(2, :)-fv(1, :),...
            fv(3, :)-fv(1, :)]);
        n1 = n1/vectorNorm3d(n1);
        cm(i,:) = cm_k;
        d0 = sqrt((x-cm_k(1)).^2+(y-cm_k(2)).^2+(z-cm_k(3)).^2);
        [dv, di] = sort(d0);
        di = di(1:inoutcheckNpoint);
        dv = dv(1:inoutcheckNpoint);
        dv2 = sqrt((x-(cm_k(1)+n1(1))).^2+(y-(cm_k(2)+n1(2))).^2+(z-(cm_k(3)+n1(3))).^2);
        dv2 = dv2(di);
        if sum(dv2)<sum(dv)
            %faces(i,:) = fliplr(faces(i, :));
            %n1 = -n1;
fprintf('            Face %i is inverted.\n', i);
        end
        
        faceNorm(i, :) = n1;
%        out = inPolyhedron2(a, cm_k + n1);
        %in = inPolyhedron2(a, cm_k - n1);
        %T = T/vectorNorm3d(T);
%         if (out~=0) 
%             faces(i,:) = fliplr(faces(i, :));
%             n1 = -n1;
%         end

        N_vertices_for_face1 = numel(f1);
        ngb_face = zeros(size(f1));
        for v_indx = 1:N_vertices_for_face1
            if v_indx < N_vertices_for_face1
                v_pair = [v_indx, v_indx+1];
            else
                v_pair = [v_indx, 1];
            end
            [Nface, f_indx2] = find_neighbor_face(i, f1(v_pair), faces);
            if numel(f_indx2)>=2
                fprintf('More than two faces share the edge made of %i and %i vertices\n', f1(v_pair(1)), f1(v_pair(2)))
                ngb_manyfaces = [ngb_manyfaces, f_indx2];
                mf = mf+1;
            elseif numel(f_indx2)==0
                fprintf('Only one face contains the edge made of %i and %i vertices\n', f1(v_pair(1)), f1(v_pair(2)))
                %ngb_manyfaces = [ngb_manyfaces, f_indx2];
                %mf = mf+1;
            else
                %fprintf('Num of f_indx2 %i for %i vertices\n', numel(f_indx2), v_indx)
                ngb_face(v_indx) = f_indx2;
            end
        end
        ngb_faces{i} = ngb_face;
    end
    a.faces = faces;
    a.vertices = vertices;
    a.ngb_faces = ngb_faces;
    a.ngb_manyfaces = unique(ngb_manyfaces);
    a.faceNorm = faceNorm;
    a.cm = cm;
fprintf('Done.\n');

end

function [vn_NF, face_index] = find_neighbor_face(f1, vn, faces)
    % f1 : face number
    % vn : shared vertices, ex. [1,3]
    % vn_NF : vertices indices of the neighboring face. 
    N = 1:size(faces, 1);
    N(f1) = [];
    faces(f1, :) = [];
    ind = sum(ismember(faces, vn), 2) == 2;
    vn_NF = faces(ind, :);
    face_index = N(ind);
end