function y = spheroid_vert_2clusters_pDist(x, pDist, p, strfactor, strp)
% [y, name, pnames, pin] = pedersen_pDist(x, pDist, p)
% Pedersen formalism to calculate scattering intensity from a given distribution function.
% Local monodisperse approximation assumed.
% Formfactor assumed as sphere scattering.
% Structure factor : Hard sphere
% pDist = [x, Dist];
% p = [I0, volfrac, back];

if nargin > 4;
    xx = pDist(:, 1);
    gDist = pDist(:, 2);
%    fn = fieldnames(strp);
%    df = strp.(fn{1});
%    dfa = strp.(fn{2});
    
%    df = strp.dspacing;
%    dfa = strp.sig;
%    normFactor = sum(gDist.*xx.^6);  % for sphere
        %Form = spheretype(x, xx(i))*xx(i)^6;  % let's discribe using volume distribution... because I don't want exaggerate small size particle...
        if isfield(x, 'q')
            qpa = x.qpa;
            q1z = x.q1z;
            q2z = x.q2z;
            q3z = x.q3z;
            q4z = x.q4z;
            Ti = x.Ti;
            Tf = x.Tf;
            Ri = x.Ri;
            Rf = x.Rf;
            GI = 1;
        else
            x = x(:);
            qpa = x;
            q1z = 0;
            q2z = 0;
            q3z = 0;
            q4z = 0;
            Ti = 1;
            Tf = 0;
            Ri = 0;
            Rf = 0;
            GI = 0;
        end
        RcRp1 = p.RcRp1;
        RcbRc1 = p.RcbRc1;
        RcRp2 = p.RcRp2;
        RcbRc2 = p.RcbRc2;
    I01 = p.I01;
    I02 = p.I02;
    normFactor1 = sum(gDist.*(xx.^6))*RcRp1^2*(4/3*pi).^2;  % for cylinder
    normFactor2 = sum(gDist.*(xx.^6))*(p.r2perR)^6*RcRp2^2*(4/3*pi).^2;  % for cylinder
    y = zeros(size(qpa));
    for i = 1:length(xx)
%        F1 = cylinder_vert_type(qpa, q1z, [xx(i), xx(i)*HperR]);
%        F2 = cylinder_vert_type(qpa, q3z, [xx(i), xx(i)*HperR]);
        F1 = dwbasph(qpa, q1z, q2z, q3z, q4z, Ti, Tf, Ri, Rf, xx(i), RcRp1, RcbRc1, [1, 0], 0);%xx(i)*RcbRc*RcRp);
        F2 = dwbasph(qpa, q1z, q2z, q3z, q4z, Ti, Tf, Ri, Rf, xx(i)*p.r2perR, RcRp2, RcbRc2, [1, 0], 0);%xx(i)*RcbRc*RcRp);
%        Struc = feval(strfactor, x, dfa*df*xx(i), xx(i)*df); % for strfactor_2Dpara
        if ~isempty(strfactor)
            Struc = feval(strfactor, x, strp, xx(i));  % for strfactor2
        else
            Struc =1;
        end
        y = y + (abs(F1).^2/normFactor1*I01+abs(F2).^2/normFactor2*I02).*Struc*gDist(i);
    end
%    y = y*p.I0/normFactor;
end
