#include "math.h"
#include "mex.h"

/* The gateway routine */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    mxArray *xxD;
    double *qp, *qz1, *qz2;
    double *y, sig1, sig2, zint1, zint2, hcor, vcor, H;
    double yt[1];
    double q1, q2, q3;
    int nn;
    int mrows, ncols;
    int mxrows, nxcols;
    int ind,m;

  if (nrhs != 10) 
    mexErrMsgTxt("Twelve inputs required.");
  if (nlhs != 1) 
    mexErrMsgTxt("One output required.");

    xxD = prhs[0];
    qp = mxGetPr(xxD);
    qz1 = mxGetPr(prhs[1]);
    qz2 = mxGetPr(prhs[2]);
    sig1 = mxGetScalar(prhs[3]);
    sig2 = mxGetScalar(prhs[4]);
    zint1 = mxGetScalar(prhs[5]);
    zint2 = mxGetScalar(prhs[6]);
    hcor = mxGetScalar(prhs[7]);
    vcor = mxGetScalar(prhs[8]);
    H = mxGetScalar(prhs[9]);
    
    mxrows = mxGetM(xxD);
    nxcols = mxGetN(xxD);

    /* Output setting */
    plhs[0] = mxCreateDoubleMatrix(mxrows, nxcols, mxREAL);
    y = mxGetPr(plhs[0]);

    for (m=0;m<(mxrows*nxcols);m++) {
        q1 = qp[m];
        q2 = qz1[m];
        q3 = qz2[m];
        corr(q1,q2,q3, sig1, sig2, zint1, zint2, hcor, vcor, H, yt);
        y[m] = yt[0];
        }
}

void corr(double qx, double qz1, 
double qz2, double sig1, double sig2, double zint1, double zint2, double hcor, double vcor, double H, double *y)
{
    int i;
    float r;
    float rmax, r_stp;
    float numpnt = 10.0;
    float numsm = 10.0;
    double corr_end=0.001;
    double ratio;
    double Cr;
    double yt[1];
    rmax = hcor*exp(log(log(1/corr_end))/2/H);
    r_stp = 2.0*3.141592/(qx+0.0000001)/numpnt;
    ratio = rmax/r_stp;
    if (ratio < 10) r_stp = rmax/numsm;
//    if (qx < 0.01) r_stp = rmax/numsm;
//    else r_stp = 2.0*3.141592/qx/numpnt;
    Cr = 0.0;
//    printf("r_stp = %f, rmax = %f\n", r_stp, rmax);
    for (r=0;r<=rmax;r=r + r_stp) {
            yt[0]=j0(qx*r);
            Cr = Cr + (exp((qz1*qz2)*sig1*sig2*exp(-1*pow((r/hcor),(2*H))-abs(zint1-zint2)/vcor))-1)*r*yt[0];
        }
    y[0] = Cr*r_stp;
}


