#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <octave/oct.h>
#include "defun-dld.h"
#include "error.h"
#include "gripes.h"
#include "oct-obj.h"
#include "utils.h"
#include "dMatrix.h"

#include <stdio.h>
#include "math.h"

double corr(double qx, double qz1, 
double qz2, double sig1, double sig2, double zint1, double zint2, double hcor, double vcor, double H)
{
    float r;
    float rmax, r_stp;
    float numpnt = 10.0;
    float numsm = 10.0;
    double corr_end=0.001;
    double ratio;
    double Cr;
    double yt;
    rmax = hcor*exp(log(log(1/corr_end))/2/H);
    r_stp = 2.0*3.141592/(qx+0.0000001)/numpnt;
    ratio = rmax/r_stp;
    if (ratio < 10) r_stp = rmax/numsm;
    Cr = 0.0;
    for (r=0;r<=rmax;r=r + r_stp) {
            yt=j0(qx*r);
            Cr = Cr + (exp((qz1*qz2)*sig1*sig2*exp(-1*pow((r/hcor),(2*H))-fabs(zint1-zint2)/vcor))-1)*r*yt;
        }
    return Cr*r_stp;
}

/* The gateway routine */
DEFUN_DLD (corr_qxy, args, nargout,
  "-*- texinfo -*-\n\
@deftypefn {Loadable Function} {[@var{x}, @var{rcond}] = } circ (@var{image},@var{[beam X, beam Y]},@var{mu_start_ang},@var{mu_end_ang})\n\
This routine for the circular averaging\n\
Using the 2D interpolation fuction\n\
@end deftypefn")

{ 
    octave_value retval;
//    int nargin = args.length();
   
    Matrix qp = args(0).matrix_value();
    Matrix qz1 = args(1).matrix_value();
    Matrix qz2 = args(2).matrix_value();
    double sig1 = args(3).double_value();
    double sig2 = args(4).double_value();
    double zint1 = args(5).double_value();
    double zint2 = args(6).double_value();
    double hcor = args(7).double_value();
    double vcor = args(8).double_value();
    double H = args(9).double_value();
    
    double q1, q2, q3;
    int mxrows = qp.rows();
    int nxcols = qp.columns();
    Matrix y(mxrows, nxcols);
    for (int m=0;m<mxrows;m++) {
	for (int n=0;n<nxcols;n++){
            q1 = qp(m,n);
            q2 = qz1(m,n);
            q3 = qz2(m,n);
            y(m,n) = corr(q1,q2,q3, sig1, sig2, zint1, zint2, hcor, vcor, H);
	}
    }
    retval = y;
    return retval;
}
